@@ -104,11 +104,40 @@ int sms_select_mem (char* mem, struct slot_range* r,
 }
 
 #include "ttyaccess.h"
-int phone_init (void) {
+#include <unistd.h>
+
+void phone_reset (void) {
+  print_verbose(0,"%s...",_("Sending reset frame"));
+  /* sending a sync frame
+   * the phone may not respond without it because it thinks that all
+   * send commands are still data from a previous run
+   */
+  tty_write("\x1a\r",2);
+  /* it might also happen that some phones are
+   * in "transparent mode" and must be switched to "RCCP mode"
+   */
+  usleep(1010*1000);
+  tty_write("+++",3);
+  usleep(1010*1000);
+  tty_write("\r",1);
+  usleep(1000*1000);
+
+  /* flush all data because we are not interested in it */
+  tty_flush();
+  print_verbose(0,"%s\n",_("done"));
+}
+
+int phone_init (int reset) {
   int retval = 1;
   char* ack;
 
-  tty_write("\r",1);
+  if (reset) { /* big reset */
+    phone_reset();
+  } else { /* small reset */
+    tty_write("\r",1);
+    usleep(1000*1000);
+    tty_flush();
+  }
   at_command_send(AT_GEN_INIT,NULL);
   ack = at_read_line();
   if (at_line_type(ack,NULL,NULL) != AT_RET_OK) {
@@ -117,8 +146,8 @@ int phone_init (void) {
     if (at_line_type(ack,NULL,NULL) != AT_RET_OK) {
       retval = 0;
     }
-    mem_realloc(ack,0);
   }
+  mem_realloc(ack,0);
   return retval;;
 }
 
@@ -192,7 +221,7 @@ int set_charset(char *charset) {
   int  retval = 0;
 
   if (str_len(charset) == 0) {
-    errexit ("%s: %s\n",_("Error"),_("no charset specified."));
+    errexit("%s\n",_("no charset specified."));
   }
   at_gen_charset(charset);
   temp = at_read_line();
@@ -209,18 +238,17 @@ void set_smsc (char* number) {
   enum return_code status;
 
   if (str_len(number) == 0) {
-    errexit("%s: %s\n",_("Error"),_("no number specified."));
+    errexit("%s\n",_("no number specified."));
   } else if (is_telephone_number(number) == 0) {
-    errexit("%s: %s\n",_("Error"),_("no valid number specified."));
+    errexit("%s\n",_("no valid number specified."));
   }
   at_gen_smsc(number);
   temp = at_read_line();
   status = at_line_type(temp,AT_GEN_SMSC,&error);
   if (status == AT_RET_OK) {
-    myprintf(0,_("SMS server number was set to %s\n"),number);
+    print_verbose(0,_("SMS server number was set to %s\n"),number);
   } else {
-    errexit("%s: %s\n",_("Error"),
-	    (status == AT_RET_ERROR) ? _("unknown cause") : error);
+    errexit("%s\n", (status == AT_RET_ERROR) ? _("unknown cause") : error);
   }
   mem_realloc(temp,0);
 }
@@ -256,7 +284,7 @@ void set_time (void) {
   at_command_send(AT_GEN_TIME,timestr);
   ack = at_read_line();
   if (at_line_type(ack,NULL,NULL) == AT_RET_OK) {
-    myprintf(0,"%s\n",_("Time was synchronized"));
+    print_verbose(0,"%s\n",_("Time was synchronized"));
   } else {
     errexit("%s\n",_("Time could not be synchronized"));
   }
@@ -519,8 +547,8 @@ char* get_gprs_class (void) {
 
 void info_misc (FILE* fd) {
   char *temp;
-  int i = 0;
-  int j;
+  int i = 0, j;
+  unsigned int area = 0, cell = 0;
   struct tm* ltime;
   char** memlist;
   unsigned int fieldsize = 0;
@@ -546,34 +574,34 @@ void info_misc (FILE* fd) {
   fields[18] = NULL;
 
   while (fields[i] != NULL) {
-    if (fieldsize < strlen(fields[i])) fieldsize = strlen(fields[i]);
+    if (fieldsize < (size_t)str_width(fields[i])) fieldsize = str_width(fields[i]);
     ++i;
   }
   fieldsize += 2;
 
   fprintf(fd,"%s:\n",_("Phone related information"));
-  fprintf(fd,"%s:%*s%s\n",fields[0],fieldsize-strlen(fields[0]),"",
+  fprintf(fd,"%s:%*s%s\n",fields[0],fieldsize-str_width(fields[0]),"",
 	  temp=get_vendor());
   mem_realloc(temp,0);
 
-  fprintf(fd,"%s:%*s%s\n",fields[1],fieldsize-strlen(fields[1]),"",
+  fprintf(fd,"%s:%*s%s\n",fields[1],fieldsize-str_width(fields[1]),"",
 	  temp=get_model());
   mem_realloc(temp,0);
 
-  fprintf(fd,"%s:%*s%s\n",fields[2],fieldsize-strlen(fields[2]),"",
+  fprintf(fd,"%s:%*s%s\n",fields[2],fieldsize-str_width(fields[2]),"",
 	  temp=get_revision());
   mem_realloc(temp,0);
 
-  fprintf(fd,"%s:%*s%s\n",fields[3],fieldsize-strlen(fields[3]),"",
+  fprintf(fd,"%s:%*s%s\n",fields[3],fieldsize-str_width(fields[3]),"",
 	  temp=get_phoneserial());
   mem_realloc(temp,0);
 
-  fprintf(fd,"%s:%*s%s%%\n",fields[4],fieldsize-strlen(fields[4]),"",
+  fprintf(fd,"%s:%*s%s%%\n",fields[4],fieldsize-str_width(fields[4]),"",
 	  temp=get_battery());
   mem_realloc(temp,0);
 
   memlist=get_charsets();
-  fprintf(fd,"%s:%*s",fields[5],fieldsize-strlen(fields[5]),"");
+  fprintf(fd,"%s:%*s",fields[5],fieldsize-str_width(fields[5]),"");
   if (memlist == NULL) {
     fprintf(fd,_("none"));
   } else {
@@ -591,7 +619,7 @@ void info_misc (FILE* fd) {
   temp=mem_alloc(81,1);
   ltime=get_time();
   if (ltime!=NULL && strftime(temp,80,"%c",ltime)) {
-    fprintf(fd,"%s:%*s%s\n",fields[6],fieldsize-strlen(fields[6]),"",temp);
+    fprintf(fd,"%s:%*s%s\n",fields[6],fieldsize-str_width(fields[6]),"",temp);
     mem_realloc(ltime,0);
   }
   mem_realloc(temp,0);
@@ -599,12 +627,12 @@ void info_misc (FILE* fd) {
 
 
   fprintf(fd,"\n%s:\n",_("SIM card related information"));
-  fprintf(fd,"%s:%*s%s\n",fields[7],fieldsize-strlen(fields[7]),"",
+  fprintf(fd,"%s:%*s%s\n",fields[7],fieldsize-str_width(fields[7]),"",
 	  temp=get_simserial());
   mem_realloc(temp,0);
 
   if ((temp=get_simid()) != NULL) {
-    fprintf(fd,"%s:%*s%s\n",fields[8],fieldsize-strlen(fields[8]),"",
+    fprintf(fd,"%s:%*s%s\n",fields[8],fieldsize-str_width(fields[8]),"",
 	    temp);
   }
   mem_realloc(temp,0);
@@ -612,47 +640,47 @@ void info_misc (FILE* fd) {
 
 
   fprintf(fd,"\n%s:\n",_("Network related information"));
-  temp=get_netstatus(&i,&j);
+  temp=get_netstatus(&area,&cell);
   if (temp!=NULL) {
-    fprintf(fd,"%s:%*s%s\n",fields[9],fieldsize-strlen(fields[9]),"",
+    fprintf(fd,"%s:%*s%s\n",fields[9],fieldsize-str_width(fields[9]),"",
 	    temp);
-    if (i) {
-      fprintf(fd,"%s:%*s%04X\n",fields[10],fieldsize-strlen(fields[10]),"",i);
-      if (j) fprintf(fd,"%s:%*s%04X\n",fields[11],fieldsize-strlen(fields[11]),"",j);
+    if (area) {
+      fprintf(fd,"%s:%*s%04X\n",fields[10],fieldsize-str_width(fields[10]),"",area);
+      if (cell) fprintf(fd,"%s:%*s%04X\n",fields[11],fieldsize-str_width(fields[11]),"",cell);
     }
   }
 
   temp=get_operator();
 
   if (temp!=NULL) {
-    fprintf(fd,"%s:%*s%s\n",fields[12],fieldsize-strlen(fields[12]),"",
+    fprintf(fd,"%s:%*s%s\n",fields[12],fieldsize-str_width(fields[12]),"",
 	    temp);
   }
   mem_realloc(temp,0);
 
   temp=get_smsc();
   if (temp!=NULL) {
-    fprintf(fd,"%s:%*s%s\n",fields[13],fieldsize-strlen(fields[13]),"",
+    fprintf(fd,"%s:%*s%s\n",fields[13],fieldsize-str_width(fields[13]),"",
 	    temp);
   }
   mem_realloc(temp,0);
 
   if ((i=get_signal(&j)) >= 0) {
-    fprintf(fd,"%s:%*s%d dBm\n",fields[14],fieldsize-strlen(fields[14]),"",
+    fprintf(fd,"%s:%*s%d dBm\n",fields[14],fieldsize-str_width(fields[14]),"",
 	    -i);
     if (j >= 0) {
       if (j <= 128) {
-	fprintf(fd,"%s:%*s<=%d,%d%%\n",fields[15],fieldsize-strlen(fields[15]),"",
+	fprintf(fd,"%s:%*s<=%d,%d%%\n",fields[15],fieldsize-str_width(fields[15]),"",
 		j/10,j%10);
       } else {
-	fprintf(fd,"%s:%*s>12,8%%\n",fields[15],fieldsize-strlen(fields[15]),"");
+	fprintf(fd,"%s:%*s>12,8%%\n",fields[15],fieldsize-str_width(fields[15]),"");
       }
     }
   }
 
   temp=get_gprs_class();
   if (temp!=NULL) {
-    fprintf(fd,"%s:%*s",fields[16],fieldsize-strlen(fields[16]),"");
+    fprintf(fd,"%s:%*s",fields[16],fieldsize-str_width(fields[16]),"");
     temp=(char *)strtok(temp,"()\",");
     while (temp!=0) {
       fprintf(fd,"%s",temp);
@@ -667,7 +695,7 @@ void info_misc (FILE* fd) {
 
   temp=get_gprs_status(NULL,NULL);
   if (temp!=NULL) {
-    fprintf(fd,"%s:%*s%s",fields[17],fieldsize-strlen(fields[17]),"",
+    fprintf(fd,"%s:%*s%s",fields[17],fieldsize-str_width(fields[17]),"",
 	    temp);
     i=get_gprs_attach();
     if (i==0) {
@@ -679,11 +707,25 @@ void info_misc (FILE* fd) {
 }
 
 void info_mem (FILE* fd, int full) {
-  fprintf(fd,"%s:\t",_("Binary files"));
+  unsigned int i = 0;
+  unsigned int fieldsize = 0;
+  char* fields[4];
+  fields[0] = _("Binary files");
+  fields[1] = _("Phonebooks");
+  fields[2] = _("SMS storages");
+  fields[3] = NULL;
+
+  while (fields[i] != NULL) {
+    if (fieldsize < (size_t)str_width(fields[i])) fieldsize = str_width(fields[i]);
+    ++i;
+  }
+  fieldsize += 2;
+
+  fprintf(fd,"%s:%*s",fields[0],fieldsize-str_width(fields[0]),"");
   file_print_memlist(fd,full);
-  fprintf(fd,"%s:\t",_("Phonebooks"));
+  fprintf(fd,"%s:%*s",fields[1],fieldsize-str_width(fields[1]),"");
   pbook_print_memlist(fd,full);
-  fprintf(fd,"%s:\t",_("SMS storages"));
+  fprintf(fd,"%s:%*s",fields[2],fieldsize-str_width(fields[2]),"");
   sms_print_memlist(fd,full);
 }
 
