@@ -223,6 +223,8 @@ enum return_code at_line_type (const char* response,
 #define AT_GEN_PASSWORD      "+CPWD" // set/change password to a lock
 #define AT_GEN_TERM_CMODE    "+CMEC" // mobile termination control mode (needed for keypad control)
 #define AT_GEN_KEYPAD        "+CKPD" // keypad control
+#define AT_GEN_HANGUP        "+CHUP" // terminate a call
+#define AT_GEN_CALLINFO      "+CLCC" // list current calls
 
 /* siemens-specific commands
  * probably only works with phones made by Siemens
@@ -238,7 +240,7 @@ enum return_code at_line_type (const char* response,
 #define AT_SIE_SMS_SLOT_READ  "^SMGR" // read SMS from slot (without marking unread as read)
 #define AT_SIE_SMS_LIST_READ  "^SMGL" // read SMS by list (without marking unread as read)
 #define AT_SIE_LOCK           "^SLCK" // set/get lock status
-#define AT_SIE_PASSWORD       "+CPWD" // set/change password to a lock
+#define AT_SIE_PASSWORD       "^SPWD" // set/change password to a lock
 #define AT_SIE_PREF_OP_READ   "^SPLR" // read an entry from the preferred operator list
 #define AT_SIE_PREF_OP_WRITE  "^SPLW" // write an entry to the preferred operator list
 #define AT_SIE_OP_LIST        "^SPLM" // read PLMN list
