@@ -14,6 +14,7 @@
 #include <helper.h>
 #include <ttyaccess.h>
 #include <gtincl.h>
+#include <pinfile.h>
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -21,10 +22,10 @@
 #include <strings.h>
 #include <errno.h>
 
-char* at_last_command;
-char* at_last_parameter;
-int   at_last_was_syntax_request;
-char* PIN;
+static char* at_last_command;
+static char* at_last_parameter;
+static int   at_last_was_syntax_request;
+static char* PIN;
 
 void at_init (char* pin) {
   at_last_command = NULL;
@@ -38,12 +39,9 @@ int at_check_line_end (const char* input, const size_t len) {
 	  (input[0] == '>' && len > 1));
 }
 
-char* at_read_line () {
+char* at_read_line_raw () {
   char* value;
   char* temp;
-  char* ctemp;
-  char* ptemp;
-  int srtemp;
   int repeat;
   int estatus;
 
@@ -57,11 +55,11 @@ char* at_read_line () {
       } else {
 	estatus = ETIMEDOUT;
       }
-      errexit("%s: %s\n",_("Error on reading from device"),strerror(estatus));
+      errexit("%s: %s\n",_("Cannot read from device"),strerror(estatus));
     } else {
       if ((temp=index(value,'\r')) != NULL ||
 	  (temp=index(value,'\n')) != NULL) {	
-	//we don't want values 13=<CR>='\r' and 10=<LF>='\n' in our strings
+	//we do not want values 13=<CR>='\r' and 10=<LF>='\n' in our strings
 	memset(temp,0,1);
 	//we do not want to return empty strings
 	//we do not accept the at_command_p as answer
@@ -76,36 +74,71 @@ char* at_read_line () {
       }
     }
   } while (repeat);
-  myprintf(2,"%s: %s\n",_("Received"),value);
+  print_message(2,_("Received"),"%s\n",value);
+  return value;
+}
 
+char* at_read_line () {
+  char* value;
+  char* temp;
+  char* ctemp;
+  char* ptemp;
+  char* pintype;
+  int srtemp;
+  
+  value = at_read_line_raw();
+  
   //PIN handling for all function is done transparently here
-  //broken, see below
   if (at_line_type(value,NULL,&temp) == AT_RET_ERROR_CME) {
     if (strstr(temp,"PIN") != NULL ||
 	strstr(temp,"PUK") != NULL) {
+      //save previous command
+      ctemp = str_dup(at_last_command);
+      ptemp = str_dup(at_last_parameter);
+      srtemp = at_last_was_syntax_request;
+      
+      /* On first PIN usage, the command line option overrides
+       * pin file usage, subsequent request are always answered
+       * by pin file.
+       */
       if (str_len(PIN) == 0) {
-	errexit("%s\n%s\n",value+12,_("Use the --pin parameter."));
+        at_syntax_request(AT_GEN_PIN);
+	pintype = at_read_line_raw();
+	if (at_line_type(pintype,AT_GEN_PIN,&temp) == AT_RET_ANSWER)
+	  mem_realloc(at_read_line_raw(),0);	
+	  PIN = pinfile_get(temp);
+	mem_realloc(pintype,0);
+      }
+      if (str_len(PIN) == 0) {
+	//fail if pin file lookup failed and nothing was specified at command line
+	ctemp = mem_realloc(ctemp,0);
+	ptemp = mem_realloc(ptemp,0);
+	if (PIN != NULL) mem_realloc(PIN,0);
+	at_line_type(value,NULL,&temp);
+	errexit("%s\n%s\n",temp,_("Use the --pin parameter or edit the pin file."));
       } else {
-	//save previous command
-	ctemp = str_dup(at_last_command);
-	ptemp = str_dup(at_last_parameter);
-	srtemp = at_last_was_syntax_request;
+        //resend command to be sure to be allowed to enter the pin
+	if (srtemp) at_syntax_request(ctemp);
+	else at_command_send(ctemp,ptemp);
+	mem_realloc(at_read_line_raw(),0);
 
+	//enter the pin
 	at_gen_pin(PIN);
-	PIN = mem_realloc(PIN,0); //prevent a loop
 	mem_realloc(value,0);
-	value = at_read_line();
-	if (at_line_type(value,AT_GEN_PIN,&temp) != AT_RET_OK) {
-	  errexit("%s: %s\n",_("Error on using pin"),temp);
-	}
+	value = at_read_line_raw();
+	if (at_line_type(value,AT_GEN_PIN,&temp) != AT_RET_OK)
+	  errexit("%s: %s\n",_("Using PIN failed"),temp);
 	mem_realloc(value,0);
 
+	//prevent a loop, use pin file for multiple usages
+	PIN = mem_realloc(PIN,0);
+
 	//resend previous command
 	if (srtemp) at_syntax_request(ctemp);
 	else at_command_send(ctemp,ptemp);
 	ctemp = mem_realloc(ctemp,0);
 	ptemp = mem_realloc(ptemp,0);
-	value = at_read_line();
+	value = at_read_line_raw();
       }
     }
   } else if (strncmp(value,"RING",4) == 0 ||
@@ -119,11 +152,11 @@ char* at_read_line () {
 void at_command_send (const char* command, const char* parmlist) {
   if (command == NULL) return;
 
-  myprintf(2,"\n%s: %s%s",_("Sending command"),AT_PREFIX,command);
+  print_message(2,_("\nSending command"),"%s%s",AT_PREFIX,command);
   if (str_len(parmlist) > 0) {
-    myprintf(2,"%s%s\n",AT_WRITE_DELIM,parmlist);
+    print_verbose(2,"%s%s\n",AT_WRITE_DELIM,parmlist);
   } else {
-    myprintf(2,"%s","\n");
+    print_verbose(2,"%s","\n");
   }
 
   if (tty_write(AT_PREFIX,sizeof(AT_PREFIX)-1) == 0 ||
@@ -132,7 +165,7 @@ void at_command_send (const char* command, const char* parmlist) {
        (tty_write(AT_WRITE_DELIM,sizeof(AT_WRITE_DELIM)-1) == 0 ||
 	tty_write(parmlist,strlen(parmlist)) == 0)) ||
       tty_write("\r",1) == 0) {
-    errexit("%s: %s\n",_("Error on sending at command"),strerror(errno));
+    errexit("%s: %s\n",_("Cannot send at command"),strerror(errno));
   } else {
     at_last_command = mem_realloc(at_last_command,0);
     at_last_command = str_dup(command);
@@ -145,13 +178,13 @@ void at_command_send (const char* command, const char* parmlist) {
 void at_syntax_request (const char* command) {
   if (command == NULL) return;
 
-  myprintf(2,"\n%s: %s%s%s\n",_("Sending command"),
+  print_message(2,_("\nSending command"),"%s%s%s\n",
 	   AT_PREFIX, command, AT_READ_SUFFIX);
   if (tty_write(AT_PREFIX,sizeof(AT_PREFIX)-1) == 0 ||
       tty_write(command,strlen(command)) == 0 ||
       tty_write(AT_READ_SUFFIX,strlen(AT_READ_SUFFIX)) == 0 ||
       tty_write("\r",1) == 0) {
-    errexit("%s: %s\n",_("Error on sending at command"),strerror(errno));
+    errexit("%s: %s\n",_("Cannot send at command"),strerror(errno));
   } else {
     at_last_command = mem_realloc(at_last_command,0);
     at_last_command = str_dup(command);
@@ -173,6 +206,7 @@ char* at_get_value (const char* command) {
   case AT_RET_OTHER:
     temp2 = at_read_line();
     if (at_line_type(temp2,NULL,NULL) == AT_RET_OK) {
+      mem_realloc(temp2,0);
       return temp;
     }
     mem_realloc(temp2,0);
@@ -265,6 +299,6 @@ enum return_code at_line_type (const char* response,
     temp = _("nothing");
     retval =  AT_RET_OTHER;
   }
-  myprintf(3,_("Value matched %s.\n"),temp);
+  print_verbose(3,_("Value matched %s.\n"),temp);
   return retval;
 }
