@@ -38,13 +38,15 @@ ucs4char_t* convert_from_system (char* input) {
    */
   while (temp[i] != 0) {
     switch (temp[i]) {
-    case 0x5c: // '\'
+    case 0x5c: // backslash is cast character
       switch (temp[i+1]) {
       case 0x6e: // 'n'
 	retval[k++] = 0x0a;
+	++i;
 	break;
-      case 0x5c: // '\'
+      case 0x5c: // backslash
 	retval[k++] = 0x5c;
+	++i;
 	break;		    
       default:
 	status = 1; //assume successful loop
