@@ -20,7 +20,13 @@
 #include <string.h>
 #include <errno.h>
 #include <iconv.h>
+#include <limits.h>
 
+#ifndef MB_LEN_MAX
+#define MB_LEN_MAX 6
+#endif
+
+static
 size_t replace_char_escape (char **inbuf, size_t *inbytesleft,
                             char **outbuf, size_t *outbytesleft)
 {
@@ -39,6 +45,8 @@ size_t replace_char_escape (char **inbuf, size_t *inbytesleft,
   *outbytesleft -= 5;
   return 0;
 }
+
+static
 size_t replace_char_questionmark (char **inbuf, size_t *inbytesleft,
 				  char **outbuf, size_t *outbytesleft)
 {
@@ -58,15 +66,16 @@ size_t replace_char_questionmark (char **inbuf, size_t *inbytesleft,
   return 0;
 }
 
+static
 size_t replace_char(enum repmode replacement_mode,
 		    char **inbuf, size_t *inbytesleft,
 		    char **outbuf, size_t *outbytesleft)
 {
   size_t retval = 0;
   switch(replacement_mode) {
-  case REPMODE_ABORT:
-    fprintf(stderr,_("%s: Unicode character 0x%x cannot be converted.\n"),_("Error"),*((ucs4char_t*)*inbuf));
-    exit(EXIT_FAILURE);
+  case REPMODE_IGNORE:
+    fprintf(stderr,_("%s: Unicode character 0x%lx cannot be converted.\n"),_("Error"),
+            (unsigned long)*((ucs4char_t*)*inbuf));
     break;
   case REPMODE_ESCAPE_CHARS:
     retval = replace_char_escape(inbuf,inbytesleft,outbuf,outbytesleft);
@@ -110,10 +119,10 @@ char* convert_from_internal (const char* to_code,
   inptr = (char*)input;
   inptr_conv = inptr;
 
-  //this should be enough even for 6 Bytes of UTF-8  
-  outsize = ucs4len(input)*6;
+  //this should be enough for every possible locale  
+  outsize = ucs4len(input)*MB_LEN_MAX;
   outsize_conv = outsize;
-  retval = mem_alloc(outsize+6,1); //not to be modified later
+  retval = mem_alloc(outsize+1,1); //not to be modified later
   outptr = retval;
   outptr_conv = retval;
 
