@@ -13,28 +13,46 @@
 #include "memtypes.h"
 #include "options.h"
 #include "timeincl.h"
+#include "gtincl.h"
 
 #include <stdlib.h>
 
 int VERBOSE_LEVEL;
-#define errexit(m, ...) { fprintf(stderr,m,__VA_ARGS__);exit(EXIT_FAILURE); }
-#define myprintf(l,m, ...) { if (l<=VERBOSE_LEVEL) fprintf(stderr,m,__VA_ARGS__); }
+#define VERBOSE_LEVEL_ERROR   0
+#define VERBOSE_LEVEL_WARNING 1
+#define VERBOSE_LEVEL_NOTICE  2
+#define print_verbose(level,m, ...) { if (VERBOSE_LEVEL >= level) fprintf(stderr,m,__VA_ARGS__); }
+#define print_message(level,key, m, ...) { print_verbose(level,"%s: ",key); print_verbose(level,m,__VA_ARGS__); }
+#define print_error(m, ...) print_message(VERBOSE_LEVEL_ERROR,_("\nError"),m,__VA_ARGS__)
+#define print_warning(m, ...) print_message(VERBOSE_LEVEL_WARNING,_("Warning"),m,__VA_ARGS__)
+#define print_notice(m, ...) print_message(VERBOSE_LEVEL_NOTICE,_("Notice"),m,__VA_ARGS__)
+#define errexit(m, ...) { print_error(m,__VA_ARGS__); exit(EXIT_FAILURE); }
 
 //from keypad.c
-void keypad_dial_number (char* number);
+int keypad_dial_number (char* number);
+//from dial.c
+void dial (char* number, int wait);
+void hangup ();
 
 //from lock.c
+int lock_get_status (const char* lock);
+int lock_enable (const char* lock, const char* password);
+int lock_disable (const char* lock, const char* password);
 void lock_toggle (const char* lock, const char* password);
 void lock_print_list (FILE* fd, unsigned int full);
 
 //from fileaccess.c
 int file_accessible_ro (char* file);
-int open_myFile_ro(char* file);
-int open_myFile_rw(char* file);
-void close_myFile(int filedes);
+int file_open_ro (char* file);
+int file_open_rw (char* file);
+void file_close (int filedes);
+unsigned int file_read_binary (char* file, char** buffer);
+#define open_myFile_ro(file) file_open_ro(file)
+#define open_myFile_rw(file) file_open_rw(file)
+#define close_myFile(filedes) file_close(filedes)
 
 //from action.c
-int phone_init (void);
+int phone_init (int reset);
 int command_echo (int enable);
 int verbose_errors (int enable);
 int pbook_select_mem (char* mem);
@@ -70,7 +88,8 @@ void file_get (char* file, char* ftype, int slot, char* pipe);
 //from s35_pbook.c
 void pbook_manage (int action, char* outfile,
 		   char** files, struct parameters* myparams);
-int pbook_get_ranges (char* mem, int write, int* min, int* max, int* nrlen, int* txtlen);
+int pbook_get_ranges (char* mem, int write, unsigned int* min,
+		      unsigned int* max, int* nrlen, int* txtlen);
 
 //from s35_sms.c
 void sms_manage (int action, char** files,
@@ -81,9 +100,10 @@ void sms_delete_slot (int smsslot);
 void sms_delete_all (struct sms_slot_data** slist);
 void sms_send_slot (int smsslot, char* smsnumber);
 void sms_send_pdu (char* pdu, int direct);
+void sms_send_pdus (char** pdu, int direct);
 void sms_send_pdu_all (struct sms_slot_data** slist, char* number, int direct);
-char* sms_pdu_create (char* file, char* text, char* number,
-		      struct smsopts* mysmsopts);
+char** sms_pdu_create (char* file, char* text, char* number,
+		       struct smsopts* mysmsopts);
 int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read);
 void sms_print (FILE* filefd, char* pipe, struct sms* data);
 void sms_print_all (char* file, char* pipe, struct sms_slot_data** slist,
