@@ -94,12 +94,15 @@ void help(char *me){
   help_print_line("get",'g',_("get something from the phone"));
   help_print_line("info",'i',_("show various information"));
   help_print_line("mem-info",0,_("show memory information"));
+  help_print_line("lock <lock>",0,_("selects a lock to change (might need option --pin)"));
+  help_print_line("enable",0,_("enable e.g. a lock"));
+  help_print_line("disable",0,_("disable e.g. a lock"));
   help_print_line("lock-info",0,_("show lock information"));
   help_print_line("set-time",0,_("synchronize time on phone"));
   help_print_line("set-smsc",0,_("set the SMSC number (requires option -n)"));
-  help_print_line("toggle-lock <lock>",0,_("changes lock status (might need option --pin)"));
-  help_print_line("dial",0,_("dials a number using remote keypad control (requires option -n)"));
-  help_print_line("reset",0,_("sends some special characters and exits"));
+  help_print_line("dial",0,_("dials a number (requires option -n)"));
+  help_print_line("hangup",0,_("hangup all active calls"));
+  help_print_line("reset",0,_("sends some special characters"));
   help_print_line("help",'h',_("print this message"));
   help_print_line("version",0,_("print the version number"));
 
