@@ -13,38 +13,47 @@
 #define HELPERS_H
 
 #include "intincl.h"
-#include <stdarg.h>
-#include <stdio.h>
+#include <stdlib.h>
 
-//gets path to home directory
-const char* get_homedir ();
-
-//returns the number of digits in num
+/* returns the number of digits in num */
 unsigned int numlen (long num);
-//requires to link with -lm
-//does not work with Cygwin (log10l() not implemented)
-/* #include <math.h> */
-/* #define numlen(zahl)  ((unsigned int)(log10l(labs(zahl))+1)) */
 
+/* returns the GSM type of address (TOA) value */
 uint8_t numtype (const char* number);
+
+/* returns 1 if every character in s is a digit else 0*/
 int is_number(const char* s);
+
+/* returns 1 if s is a telephone number else 0 */
 int is_telephone_number (const char* s);
+
+/* converts a hexadezimal string with a given length to an int */
 unsigned int hexstr2int(char *hexstring, unsigned int length);
 
+
+/* (re)allocate memory */
 void* mem_alloc (size_t size, short zero_it);
 void* mem_realloc (void* oldpointer, size_t size);
 
+/* duplicate a c string */
 char* str_dup (const char* input);
 char* strn_dup (const char* input, size_t insize);
 
-/* strlen() does not seem to check for strlen(NULL) and crashes
- */
+/* strlen() does not seem to check for strlen(NULL) and crashes */
 size_t str_len(const char *s);
 
-/* reverse compare two string
- * returns 1 if the last n characters match else returns 0
- * s1 and s2 can be shorter than n
+/** calculate the printed string width
+ * @param s string in ISO-8859-? or UTF-8
+ * @return amount of printed characters
  */
-int strn_reverse_compare (const char* s1, const char* s2, size_t n);
+int str_width (const char* s);
+
+/** split a string
+ * @param str string to split up (it gets modified!)
+ * @param split at what points in str to split
+ * 
+ * @return return the dynamically allocated array ([0] = str)
+ */
+char** str_split (char* str, char* split);
 
 #endif
