@@ -11,6 +11,7 @@
 
 #include "memtypes.h"	 
 #include "common.h"
+#include "options.h"
 #include "helper.h"
 #include "atcommand.h"
 #include "gtincl.h"
@@ -18,84 +19,118 @@
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
+#include <stdarg.h>
 
-unsigned int roundup2 (unsigned int d) {
+static unsigned int roundup2 (unsigned int d) {
   return ((d/2) + (d%2));
 }
 
-struct slot_range** file_get_types (int mode) {
+static char* file_types_get_string (int action) {
+  char* ausgabe = get_vendor();
+  char* command;
+
+  if (ausgabe == NULL || strcasecmp(ausgabe,"SIEMENS") != 0) return NULL;
+
+  ausgabe = mem_realloc(ausgabe,0);
+  if (action == SCMXX_ACTION_GET) command = AT_SIE_BIN_READ;
+  else command = AT_SIE_BIN_WRITE;
+
+  at_command_send(command,"?");
+  return at_get_value(command);
+}
+
+static unsigned int file_type_count (const char* str) {
+  int count = 0;
+  char* start;
+  char* end;
+
+  if (str == NULL) return 0;
+  //count the number of entries
+  while ((start = strchr(str,(int)'"')) != NULL &&
+	 (end = strchr(start+1,(int)'"')) != NULL) {
+    ++count;
+    str = end+1;
+  }
+  return count;
+}
+
+static struct slot_range** file_type_filter (struct slot_range** input, int action) {
+  char* vendor = get_vendor();
+  char* model = get_model();
+  unsigned int i = 0;
+
+  if (strcasecmp(vendor,"SIEMENS") == 0) {
+    if (strcasecmp(model,"ME45") == 0 ||
+	strcasecmp(model,"S45") == 0 ||
+	strcasecmp(model,"S45i") == 0) {
+      for (; input[i] != NULL; ++i) {
+	/* actually, you should not access midi slot 4 with S45 or ME45 because
+	 * of a firmware bug:
+	 * no data loss occurs but the phone thinks, a new music file arrived
+	 * although this is not true.
+	 */
+	if (action != SCMXX_ACTION_SEND &&
+	    strcmp(input[i]->name,"mid") == 0 &&
+	    input[i]->max >= 4)
+	  input[i]->max = 3;
+      }
+    } else if (strcasecmp(model,"S55") == 0) {
+      for (; input[i] != NULL; ++i) {
+	/* S55 stalls on bmp slots 3 and 4, obviously not present or very buggy
+	 */
+	if (action != SCMXX_ACTION_GET &&
+	    strcmp(input[i]->name,"bmp") == 0 &&
+	    input[i]->max >= 2)
+	  input[i]->max = 2;
+      }
+    }
+  }
+  return input;
+}
+
+struct slot_range** file_types_get (int action) {
   struct slot_range** retval;
   char* ausgabe;
-  char* command;
   char* temp;
   char* start;
   char* end;
   int count = 0;
+  int status;
 
-  ausgabe=get_vendor();
-  if (ausgabe != NULL &&
-      !strcasecmp(ausgabe,"SIEMENS")) {
-    ausgabe = mem_realloc(ausgabe,0);
-    if (mode == 0) {
-      command = AT_SIE_BIN_READ;
-    } else {
-      command = AT_SIE_BIN_WRITE;
-    }
-    at_command_send(command,"?");
-    ausgabe = at_get_value(command);
-    /* ^SBNR: ("bmp",(0-2)),("mid",(0-10)),("vcf",(0-500)),("vcs",(1-500))
-     * ^SBNW: ("bmp",(0-4)),("mid",(0-10)),("vcf",(0-500)),("vcs",(0-500)),("t9d",(0))
-     */
-    if (ausgabe != NULL) {
-      //count the number of entries
-      temp = ausgabe;
-      while ((start = index(temp,'"')) != NULL &&
-	     (end = index(start+1,'"')) != NULL) {
-	++count;
-	temp = end+1;
-      }
-      retval = mem_alloc(sizeof(*retval)*(count+1),0);
+  ausgabe = file_types_get_string(action);
+  /* ^SBNR: ("bmp",(0-2)),("mid",(0-10)),("vcf",(0-500)),("vcs",(1-500))
+   * ^SBNW: ("bmp",(0-4)),("mid",(0-10)),("vcf",(0-500)),("vcs",(0-500)),("t9d",(0))
+   */
+
+  if (ausgabe == NULL) return NULL;
+  retval = mem_alloc(sizeof(*retval)*(file_type_count(ausgabe)+1),0);
       
-      //assign the entries;
-      count = 0;
-      temp = ausgabe;
-      while ((start = index(temp,'"')) != NULL &&
-	     (end = index(start+1,'"')) != NULL) {
-	++start;
-	if (end-start > 3) {
-	  errexit("%s: %s\n",_("Error"), _("Buffer too small."));
-	}
-	retval[count] = mem_alloc(sizeof(**retval),0);
-	strncpy(retval[count]->name,start,end-start);
-	retval[count]->name[end-start]=0;
-	if ((start = index(end+1,'(')) != NULL ||
-	    index(start+1,')') != NULL) {
-	  ++start;
-	  retval[count]->min = atoi(start);
-	  do {
-	    ++start;
-	  } while (*start != '-' && *start != ')');
-	  if (*start == '-') {
-	    retval[count]->max = atoi(start+1);
-	  } else {
-	    retval[count]->max = retval[count]->min;
-	  }
-	} else {
-	  retval[count]->min = -1;
-	  retval[count]->max = -1;	  
-	}
-	++count;
-	temp = end+1;
-      }
+  //assign the entries;
+  temp = ausgabe;
+  count = 0;
+  while ((start = strchr(temp,(int)'"')) != NULL &&
+	 (end = strchr(start+1,(int)'"')) != NULL)
+  {
+    temp = end+1;
+
+    retval[count] = mem_alloc(sizeof(**retval),0);
+    status = sscanf(start,"\"%4[^\"]\",(%d-%d)",retval[count]->name,
+		    &retval[count]->min,&retval[count]->max);
+    if (status == 3) {
+      ++count;
     } else {
-      retval = mem_alloc(sizeof(*retval),0);
+      status = sscanf(start,"\"%4[^\"]\",(%d)",retval[count]->name,
+		      &retval[count]->min);
+      if (status == 2)
+      {
+	retval[count]->max = retval[count]->min;
+	++count;
+      }
     }
-  } else {
-    retval = mem_alloc(sizeof(*retval),0);
   }
-  retval[count]=NULL;
+  retval[count] = NULL;
   mem_realloc(ausgabe,0);
-  return retval;
+  return file_type_filter(retval,action);
 }
 
 struct file_mem_list {
@@ -180,8 +215,8 @@ void file_print_memlist (FILE* fd, unsigned int full) {
     fd = stdout;
   }
 
-  rrange = file_get_types(0);
-  wrange = file_get_types(1);
+  rrange = file_types_get(SCMXX_ACTION_GET);
+  wrange = file_types_get(SCMXX_ACTION_SEND);
   memlist = struct_file_mem_list_init(rrange,wrange);
   mem_realloc(rrange,0);
   mem_realloc(wrange,0);
@@ -297,9 +332,13 @@ char** pb_get_types (void) {
      strcasecmp(ausgabe,"SIEMENS") == 0) {
     command = AT_SIE_PB_SELECT;
     mem_realloc(ausgabe,0);
-    at_command_send(AT_SIE_PB_VCF_READ,AT_READ_SUFFIX);
-    ausgabe = at_get_value(AT_SIE_PB_VCF_READ);
-    if (ausgabe != NULL) has_mem_vcf = 1;
+    ausgabe = get_model();
+    if (strcmp(ausgabe,"A65") != 0) { /* A65 stalls on this command, work-around */
+      at_command_send(AT_SIE_PB_VCF_READ,AT_READ_SUFFIX);
+      mem_realloc(ausgabe,0);
+      ausgabe = at_get_value(AT_SIE_PB_VCF_READ);
+      if (ausgabe != NULL) has_mem_vcf = 1;
+    }
   } else {
     command = AT_GEN_PB_SELECT;
   }
@@ -335,8 +374,8 @@ void pbook_print_memlist (FILE* fd, unsigned int full) {
   char* entry[columns];
 
   unsigned int i = 0;
-  int min;
-  int max;
+  unsigned int min;
+  unsigned int max;
   int nrlen;
   int txtlen;
   int writable;
@@ -395,7 +434,7 @@ void pbook_print_memlist (FILE* fd, unsigned int full) {
       writable = pbook_get_ranges(memlist[m],1,&min,&max,&nrlen,&txtlen);
       if (writable == 0) {
 	if(!pbook_get_ranges(memlist[m],0,&min,&max,&nrlen,&txtlen)) {
-	  myprintf(1,_("Warning: could not get limits of phonebook %s.\n"),memlist[m]);
+	  print_warning(_("could not get limits of phonebook %s.\n"),memlist[m]);
 	  ++m;
 	  continue;
 	}
