@@ -35,7 +35,7 @@ int pbook_phone_write_entry (struct pbook_phone_entry* entry) {
   int retval = 1;
 
   at_gen_phonebook_write(entry->slot,entry->number,
-			 entry->numtype,entry->text);
+			 entry->numtype,(char*)entry->text);
   temp = at_read_line();
   if (at_line_type(temp,NULL,NULL) != AT_RET_OK) retval = 0;
   mem_realloc(temp,0);
@@ -44,35 +44,35 @@ int pbook_phone_write_entry (struct pbook_phone_entry* entry) {
 
 int pbook_phone_write (char* mem, struct pbook_phone_entry** entries) {
   unsigned int min,max;
-  unsigned int nlen,tlen;
+  int nlen,tlen;
   unsigned int i = 0;
   unsigned int count = 0;
 
   if (!pbook_get_ranges(mem,1,&min,&max,&nlen,&tlen)) {
-    errexit(_("Error on getting limits of phonebook %s.\n"),mem);
+    errexit(_("getting limits of phonebook %s failed.\n"),mem);
   }
 
   for (i = 0; entries[i] != NULL; ++i) {
     if (min > entries[i]->slot || entries[i]->slot > max) {
-      errexit (_("%s: slot %d is out of range %d-%d.\n"),_("Error"),entries[i]->slot,min,max);
+      errexit (_("slot %d is out of range %d-%d.\n"),entries[i]->slot,min,max);
     }
   }
   count = i;
   if (count > max-min+1) {
-    errexit("%s: too many entries (max. %d).\n",_("Error"),max-min+1);
+    errexit(_("too many entries (max. %d).\n"),max-min+1);
   }
 
   if (count > 1) {
-    myprintf(0,_("Updating entries %s(%d-%d)...\n"),mem,min,max);
+    print_verbose(0,_("Updating entries %s(%d-%d)...\n"),mem,min,max);
   } else {
-    myprintf(0,_("Updating one entry in slot %d of memory %s..."),entries[0]->slot,mem);    
+    print_verbose(0,_("Updating one entry in slot %d of memory %s..."),entries[0]->slot,mem);    
   }
   for (i = 0; entries[i] != NULL; ++i) {
     if (pbook_phone_write_entry(entries[i]) == 0) return 0;
-    if (count > 1) myprintf(0," %d",entries[i]->slot);
+    if (count > 1) print_verbose(0," %d",entries[i]->slot);
   }
-  if (count > 1) myprintf(0,"%s","\n");
-  myprintf(0,"%s\n",_("done"));
+  if (count > 1) print_verbose(0,"%s","\n");
+  print_verbose(0,"%s\n",_("done"));
   return 1;
 }
 
@@ -116,7 +116,7 @@ int pbook_phone_str2entry (char* line,
   entry->slot = atoi(slot);
   entry->number = strn_dup(number,index(number,'"')-number);
   entry->numtype = atoi(numtype);
-  entry->text = strn_dup(text,rindex(text,'"')-text);
+  entry->text = (unsigned char*)strn_dup(text,rindex(text,'"')-text);
   return 1;
 }
 
@@ -176,7 +176,7 @@ pbook_phone_get_range (char* mem,
 
   ausgabe = at_read_line();
   linestatus = at_line_type(ausgabe,command,&temp);
-  if (linestatus&AT_RET_ERROR) errexit(_("Error on reading from phonebook %s: %s\n"),mem,temp);
+  if (linestatus&AT_RET_ERROR) errexit(_("reading from phonebook %s failed: %s\n"),mem,temp);
 
   retval = mem_alloc((range+1)*sizeof(*retval),0);
   while ((linestatus == AT_RET_ANSWER ||
@@ -185,25 +185,25 @@ pbook_phone_get_range (char* mem,
     buffer = mem_alloc(sizeof(**retval),0);
     struct_pbook_phone_entry_init(buffer);
     status = pbook_phone_str2entry(temp,buffer);
-    if (status <= 0) errexit(_("Error in parsing entry: %s\n"), temp);
+    if (status <= 0) errexit(_("parsing entry failed: %s\n"), temp);
 
     if (enable_slot_range_test) {
-      if (buffer->slot > slot_max) errexit(_("%s: phone returned unexpected entry %u\n"),_("Error"),buffer->slot);
+      if (buffer->slot > slot_max) errexit(_("phone returned unexpected entry %u\n"),buffer->slot);
       for (; i+slot_min < buffer->slot; ++i) {
 	retval[i] = mem_alloc(sizeof(**retval),0);
 	struct_pbook_phone_entry_init(retval[i]);
 	retval[i]->slot = i+slot_min;
-	if (VERBOSE_LEVEL < 2 && print_counter) myprintf(0," %d",retval[i]->slot);
+	if (VERBOSE_LEVEL < 2 && print_counter) print_verbose(0," %d",retval[i]->slot);
       }
     }
     retval[i] = buffer;
     if (strcasecmp(mem,VCARD_PHONEBOOK_NAME) == 0) retval[i]->slot = slot_min;
-    if (VERBOSE_LEVEL < 2 && print_counter) myprintf(0," %d",retval[i]->slot);
+    if (VERBOSE_LEVEL < 2 && print_counter) print_verbose(0," %d",retval[i]->slot);
 
     mem_realloc(ausgabe,0);
     ausgabe = at_read_line();
     linestatus = at_line_type(ausgabe,command,&temp);
-    if (linestatus&AT_RET_ERROR) errexit(_("Error on reading from phonebook %s: %s\n"),mem,temp);
+    if (linestatus&AT_RET_ERROR) errexit(_("reading from phonebook %s failed: %s\n"),mem,temp);
     ++i;
   }
   if (enable_slot_range_test) {
@@ -211,7 +211,7 @@ pbook_phone_get_range (char* mem,
       retval[i] = mem_alloc(sizeof(**retval),0);
       struct_pbook_phone_entry_init(retval[i]);
       retval[i]->slot = i+slot_min;
-      if (VERBOSE_LEVEL < 2 && print_counter) myprintf(0," %d",retval[i]->slot);
+      if (VERBOSE_LEVEL < 2 && print_counter) print_verbose(0," %d",retval[i]->slot);
     }
   }
   retval[i] = NULL; //termination of list
@@ -221,23 +221,25 @@ pbook_phone_get_range (char* mem,
 // number of requested number during a search
 #define search_delta 5
 
-int pbook_phone_find_empty (char* mem, int startslot) {
-  int min = -1, max = -1;
-  int tempmax;
+int pbook_phone_find_empty (char* mem, unsigned int startslot) {
+  unsigned int min = -1, max = -1;
+  unsigned int tempmax;
   int retval = -1;
   unsigned int i;
   struct pbook_phone_entry** temp;
 
   //reading available indexes
   if (!pbook_get_ranges(mem,0,&min,&max,NULL,NULL)) {
-    errexit(_("Error on getting limits of phonebook %s.\n"),mem);
+    errexit(_("getting limits of phonebook %s failed.\n"),mem);
   }
-  if ((min == 0 && max == 0) || startslot > max) {
+  if ((min == (unsigned int)0 && max == (unsigned int)0) ||
+      startslot > max)
+  {
     //emty phonebook
     return -1;
   }
 
-  myprintf(0,"%s\n",_("Trying to find an empty slot..."));
+  print_verbose(0,"%s\n",_("Trying to find an empty slot..."));
   if (startslot < min) startslot = min;
   for (;startslot <= max && retval < 0; startslot += search_delta ) {
     tempmax = startslot+search_delta-1;
@@ -245,10 +247,10 @@ int pbook_phone_find_empty (char* mem, int startslot) {
     temp = pbook_phone_get_range(mem,startslot,tempmax,0);
     for (i=0; temp[i] != NULL; ++i) {
       if (str_len(temp[i]->number) == 0 &&
-	  str_len(temp[i]->text) == 0 &&
+	  str_len((char*)temp[i]->text) == 0 &&
 	  retval < 0) {
 	retval = temp[i]->slot;
-	myprintf(0,_("Detected empty slot %d\n"),retval);
+	print_verbose(0,_("Detected empty slot %d\n"),retval);
       }
       struct_pbook_phone_entry_delete(temp[i]);
     }
@@ -266,7 +268,7 @@ int pbook_phone_find_empty (char* mem, int startslot) {
 struct pbook_phone_entry**
 pbook_phone_get (char* mem) {
   unsigned cur_min, cur_max;
-  int min = -1, max = -1;
+  unsigned int min = -1, max = -1;
   struct pbook_phone_entry** retval;
   struct pbook_phone_entry** temp;
   unsigned int i; //index in temp
@@ -274,13 +276,13 @@ pbook_phone_get (char* mem) {
 
   //reading available indexes
   if (!pbook_get_ranges(mem,0,&min,&max,NULL,NULL)) {
-    errexit(_("Error on getting limits of phonebook %s.\n"),mem);
+    errexit(_("getting limits of phonebook %s failed.\n"),mem);
   }
   if (min == 0 && max == 0) {
     //emty phonebook
     return NULL;
   }
-  myprintf(0,_("Receiving phonebook entries %s(%d-%d)...\n"),
+  print_verbose(0,_("Receiving phonebook entries %s(%d-%d)...\n"),
 	   mem,min,max);
   //4 times as big due to phonebook vcf
   retval = mem_alloc(((4*(max-min+1))+1)*sizeof(*retval),0);
@@ -293,6 +295,6 @@ pbook_phone_get (char* mem) {
     while (temp[i] != NULL) retval[k++] = temp[i++];
     retval[k] = NULL;    
   }
-  myprintf(0,"\n%s\n",_("done"));
+  print_verbose(0,"\n%s\n",_("done"));
   return retval;
 }
