@@ -53,21 +53,16 @@ void file_transfer_manage (int   action, char** files,
     errexit ("%s\n",_("You must define either a valid slot, nothing or \"all\"."));
   }
 
-  range = NULL;
-  if (action == SCMXX_ACTION_GET) {
-    range = file_get_types(0);
-  } else {
-    range = file_get_types(1);
-  }
-  if (range[i] == NULL) {
-    errexit("%s: %s\n",_("Error"),_("this phone does not support file exchange."));
+  range = file_types_get(action);
+  if (range == NULL || range[0] == NULL) {
+    errexit("%s\n",_("this phone does not support file exchange."));
   }
   while (range[i] != NULL &&
 	 strcmp(range[i]->name,mem)) {
     ++i;
   }
   if (range[i] == NULL) {
-    errexit("%s: type %s is not supported.\n",_("Error"),mem);
+    errexit("type %s is not supported.\n",mem);
   }
   if (slot >= SCMXX_SLOT_BINARY_MIN &&
       (range[i]->min > slot || slot > range[i]->max)) { // ... test if slot is correctly set
@@ -96,7 +91,7 @@ void file_transfer_manage (int   action, char** files,
   case SCMXX_ACTION_SEND: //sending
     if (slot != SCMXX_SLOT_ALL) {
       // get the detection range (read, not write)...
-      drange = file_get_types(0);
+      drange = file_types_get(SCMXX_ACTION_SEND);
       // ...find the mem type in it...
       while (drange[j] != NULL &&
 	     strcmp(drange[j]->name,mem)) {
@@ -129,7 +124,7 @@ void file_transfer_manage (int   action, char** files,
       current = slot; // ...or the defined slot
     }
     while (files[k] != NULL && current <= range[i]->max) {
-      myprintf(0,_("Using slot %d\n"),current);
+      print_verbose(0,_("Using slot %d\n"),current);
       file_send(files[k],mem,current);
       if (k == 0) {
 	if (enable_search == 0 && slot != SCMXX_SLOT_ALL) {
@@ -152,7 +147,7 @@ void file_transfer_manage (int   action, char** files,
 	  if (current < 0) {
 	    errexit("%s\n",_("No free slot found."));
 	  } else {
-	    myprintf(0,_("Detected empty slot %d\n"),current);
+	    print_verbose(0,_("Detected empty slot %d\n"),current);
 	  }
 	}
       }
@@ -192,7 +187,7 @@ void file_delete (char* ftype, int slot) {
   at_sie_binary_write(ftype,slot,0,0);
   ausgabe = at_read_line();
   if (!strcmp(ausgabe,"OK")) {
-    myprintf(0,_("%s %d deleted.\n"),ftype,slot);
+    print_verbose(0,_("%s %d deleted.\n"),ftype,slot);
   } else {
     errexit(_("%s %d NOT deleted, something went wrong: %s\n"),ftype,slot,ausgabe);
   }
@@ -205,7 +200,7 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
   int i;
   enum return_code status;
     
-  myprintf(0,"%s\n",_("Trying to find an empty slot..."));
+  print_verbose(0,"%s\n",_("Trying to find an empty slot..."));
   i = minimum;
   while (i <= maximum) {
     at_sie_binary_read(ftype,i);
@@ -216,10 +211,10 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
       mem_realloc(ausgabe,0);
       return i;
     case AT_RET_ERROR:
-      errexit("\n%s: %s\n",_("Error"),_("unknown cause"));
+      errexit("%s\n",_("unknown cause"));
     case AT_RET_ERROR_CME:
     case AT_RET_ERROR_CMS:
-      errexit("\n%s: %s\n",_("Error"),content);
+      errexit("%s\n",content);
     default: 
       do {
 	mem_realloc(ausgabe,0);
@@ -230,9 +225,9 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
       if (status == AT_RET_OK) {
 	++i;
       } else if (status == AT_RET_ERROR) {
-	errexit("\n%s: %s\n",_("Error"),_("unknown cause"));
+	errexit("%s\n",_("unknown cause"));
       } else {
-	errexit("\n%s: %s\n",_("Error"),content);
+	errexit("%s\n",content);
       }
       mem_realloc(ausgabe,0);
       break;
@@ -250,136 +245,108 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
 #define FILE_PDUSIZE_MIN_S45 6 //min for S45/ME45 v10
 
 #define FILE_PDUSIZE 175
+
 void file_send (char* file, char* ftype, int slot) {
+  char* data;
+  unsigned int count = file_read_binary(file,&data);
+
+  unsigned int pcount = (count/FILE_PDUSIZE);
+  unsigned int i = 1;
+  unsigned int k;
+  unsigned int psize = FILE_PDUSIZE;
+  char buffer[(FILE_PDUSIZE*2)+1];
   char* ack;
+  char* vendor = get_vendor();
+  char* model = get_model();
+  unsigned int strategy = 0;
 
-  char transfer[(2*FILE_PDUSIZE)+1];
-  char buffer[(2*FILE_PDUSIZE)+1];
-  int packetnum = 1;
-  int lastsent = 0;
+  if (count%FILE_PDUSIZE) ++pcount;
 
-  int myfd;
-  int i;
-  unsigned char data;
-  char datastr[3];
+  /* we need different stategies here
+   * 0: equal packet size except last packet (S55)
+   * 1: try to split the last two packets equally (S45/ME45)
+   * It does not matter for other models, see above for default.
+   */
+  if (strcasecmp(vendor,"SIEMENS") == 0 &&
+      (strcasecmp(model,"C45") == 0 || /* not sure if needed but works with this mode */
+       strcasecmp(model,"ME45") == 0 ||
+       strcasecmp(model,"S45") == 0 ||
+       strcasecmp(model,"S45i") == 0)) {
+    strategy = 1;
+  }
+  mem_realloc(vendor,0);
+  mem_realloc(model,0);
 
-  memset(transfer,0,sizeof(transfer));
-  memset(buffer,0,sizeof(buffer));
-  memset(datastr,0,sizeof(datastr));
-
-  //trying to open the file
-  myfd = open_myFile_ro(file);
-    
-  //filling buffer 1 (transfer buffer)
-  for(i=0;i<FILE_PDUSIZE;i++){
-    if (read(myfd,&data,1)!=1) {
+  print_verbose(0,"%s\n",_("File transfer..."));
+  for (; i <= pcount; ++i) {
+    /* split the transfer size between the last two packets
+     * make the first one bigger
+     */
+    switch (strategy) {
+    default:
+    case 0:
+      if (psize > count) psize = count;
+      break;
+    case 1:
+      if (count < 2*FILE_PDUSIZE && count > FILE_PDUSIZE)
+	psize = count/2 + (count%2); /* second last packet */
+      else if (count < FILE_PDUSIZE)
+	psize = count; /* last packet */
       break;
     }
-    sprintf(transfer+(i*2),"%02x",data);
-  }
-  //filling buffer 2
-  if (strlen(transfer) == sizeof(transfer)-1) {
-    for(i=0;i<FILE_PDUSIZE;i++){
-      if (read(myfd,&data,1)!=1) {
-	break;
-      }
-      sprintf(buffer+(i*2),"%02x",data);
-    }
-  }
 
-  myprintf(0,"%s\n",_("File transfer..."));
-  while (strlen(transfer)!=0) {
-    //starting packet transfer
-    if (strlen(buffer) > 0) {
-      at_sie_binary_write(ftype,slot,packetnum,packetnum+1);
-    } else {
-      at_sie_binary_write(ftype,slot,packetnum,packetnum);
+    /* filling buffer */
+    memset(buffer,0,sizeof(buffer));
+    for (k = 0; k < psize; ++k) {
+      sprintf(buffer+(k*2),"%02x",((uint8_t*)data)[k]&0xFF);
     }
-    myprintf(0,"%s",_("Waiting for data request..."));
+    data += k;
+    count -= k;
+
+    //starting packet transfer
+    at_sie_binary_write(ftype,slot,i,pcount);
+    print_verbose(0,"%s",_("Waiting for data request..."));
     ack = at_read_line();
 
-    if (!strncmp(ack,"> ",2)) {
-      mem_realloc(ack,0);
-
-      if (strlen(buffer) == 2*FILE_PDUSIZE ||
-	  strlen(buffer) == 0) {
-	//either this is the last package and/or
-	//we don't have to split it up
-	lastsent = strlen(transfer)/2;
-      } else {
-	//split the transfer size between the last two packets
-	lastsent = (strlen(transfer) + strlen(buffer))/4;
-	//make the first one bigger
-	lastsent += ((strlen(transfer) + strlen(buffer))/2)%2;
-      }
-
-      myprintf(2,"\n%s: %s (%d)",_("Buffer1 content"),transfer,strlen(transfer));
-      myprintf(2,"\n%s: %s (%d)",_("Buffer2 content"),buffer,strlen(buffer));
-      if (VERBOSE_LEVEL) {
-	myprintf(1,_("\nSending %d Bytes in packet %d\n"),lastsent,packetnum);
-      } else {
-	myprintf(0,"%s",_("Sending data..."));
-      }
-      //sending hex data
-      if (tty_write_data(transfer,2*lastsent) == -1) {
-	close(myfd);
-	errexit(_("Error on sending data: %s\n"), strerror(errno));
-      }
-      ack = at_read_line();
-      if (!strcmp(ack,"OK")) {
-	myprintf(0,_("Packet %d sent\n"),packetnum);
-      } else {
-	close(myfd);
-	if (!strcmp(ack,"+CME ERROR: INV CHAR IN TEXT")) {
-	  errexit("%s: %s\n",_("Error"),_("Wrong data format"));
-	} else {
-	  errexit("\n\"%s\"\n",ack);
-	}
-      }
-      mem_realloc(ack,0);
-
-      //now enter the loop if we have to
-      if (strlen(buffer) == 2*FILE_PDUSIZE) {
-	strcpy(transfer,buffer);
-	for (i=0;i<FILE_PDUSIZE;i++) {
-	  if (read(myfd,&data,1)!=1) {
-	    break;
-	  }
-	  sprintf(&buffer[i*2],"%02x",data);
-	}
-	packetnum++;
-      } else {
-	if (strlen(buffer) > 0) {
-	  strcpy(transfer,&transfer[2*lastsent]);
-	  strcpy(&transfer[strlen(transfer)],buffer);
-	  memset(buffer,0,sizeof(buffer));
-	  packetnum++;
-	} else {
-	  memset(transfer,0,1);
-	}
-      }
-    } else {
-      if (!strcmp(ack,"+CME ERROR: INVALID INDEX")) {
-	mem_realloc(ack,0);
-	myprintf(0,"%s\n",_("Packet buffer seems to be filled with something else, clearing..."));
+    if (strncmp(ack,"> ",2) != 0) {
+      if (strcmp(ack,"+CME ERROR: INVALID INDEX") == 0) {
+	print_verbose(0,"%s\n",_("Packet buffer seems to be filled with something else, clearing..."));
 	at_sie_binary_write(ftype,slot,-1,-1);
-	ack = at_read_line();
-	if (strcmp(ack,"OK")) {
-	  errexit("%s\n",ack);
-	}
 	mem_realloc(ack,0);
+	ack = at_read_line();
+	if (strcmp(ack,"OK") != 0)
+	  errexit("%s\n",ack);
       } else {
-	close(myfd);
 	errexit("\n\"%s\"\n",ack);
       }
     }
+    mem_realloc(ack,0);
+
+
+    if (VERBOSE_LEVEL) {
+      print_verbose(1,_("\nSending %d Bytes in packet %d\n"),psize,i);
+    } else {
+      print_verbose(0,"%s",_("Sending data..."));
+    }
+
+    //sending hex data
+    if (tty_write_data(buffer,2*psize) == -1)
+      errexit(_("Failed to send data: %s\n"), strerror(errno));
+    ack = at_read_line();
+    if (strcmp(ack,"OK") == 0) {
+      print_verbose(0,_("Packet %d sent\n"),i);
+    } else if (strcmp(ack,"+CME ERROR: INV CHAR IN TEXT") == 0) {
+      errexit("%s\n",_("Wrong data format"));
+    } else {
+      errexit("\n\"%s\"\n",ack);
+    }
+    mem_realloc(ack,0);
   }
-  if (packetnum) {
-    myprintf(0,"%s\n",_("File transfer complete."));
+  if (pcount) {
+    print_verbose(0,"%s\n",_("File transfer complete."));
   } else {
-    myprintf(0,"%s\n",_("Nothing to transfer."));
+    print_verbose(0,"%s\n",_("Nothing to transfer."));
   }
-  close(myfd);
 }
 
 void file_get (char* file, char* ftype, int slot, char* pipe) {
@@ -395,7 +362,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
   memset(datastr,0,sizeof(datastr));
 
   at_sie_binary_read(ftype,slot);
-  myprintf(0,_("Slot %d...\n"),slot);
+  print_verbose(0,_("Slot %d...\n"),slot);
   do{
     ausgabe = at_read_line();
     //here we have to check not only for an output of OK, but because of
@@ -404,7 +371,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
     if (!(strcmp(ausgabe,"OK")) ||
 	!(strcmp(ausgabe,"+CME ERROR: unknown"))) {
       mem_realloc(ausgabe,0);
-      myprintf(0,"%s",_("Empty slot: nothing to get\n"));
+      print_verbose(0,"%s",_("Empty slot: nothing to get\n"));
       close_myFile(myfd);
       return;
     } else if (strstr(ausgabe,"ERROR")!=NULL) {
@@ -421,7 +388,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
       if (str_len(pipe) && pipefd==NULL) {
 	pipefd=popen(pipe,"w");
       }
-      myprintf(0,_("Receiving packet %d of %d...\n"),packetnum,maxpackets);
+      print_verbose(0,_("Receiving packet %d of %d...\n"),packetnum,maxpackets);
       mem_realloc(ausgabe,0);
       ausgabe = at_read_line();
       for (i=0;i<strlen(ausgabe);i+=2) {	
@@ -432,14 +399,14 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
 	    if (str_len(pipe) && pipefd!=NULL) {
 	      pclose(pipefd);
 	    }
-	    errexit(_("Error on writing to %s: %s\n"),file, strerror(errno));
+	    errexit(_("writing to file \"%s\" failed: %s\n"),file, strerror(errno));
 	  }
 	}
 	if (str_len(pipe) && pipefd!=NULL) {
 	  if (fwrite(&data,1,1,pipefd)!=1) {
 	    close_myFile(myfd);
 	    pclose(pipefd);
-	    errexit(_("Error on writing to pipe \"%s\".\n"),pipe);
+	    errexit(_("writing to pipe \"%s\" failed.\n"),pipe);
 	  }
 	}
       }
@@ -448,13 +415,13 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
   } while (packetnum!=maxpackets);
   close_myFile(myfd);
   if (str_len(pipe) && pipefd!=NULL && pclose(pipefd)==-1) {
-    close(myfd);
-    errexit(_("Error on closing pipe \"%s\".\n"),pipe);
+    file_close(myfd);
+    errexit(_("closing pipe \"%s\" failed.\n"),pipe);
   }
   ausgabe = at_read_line();
   if (!(strcmp(ausgabe,"OK"))) {
     mem_realloc(ausgabe,0);
-    myprintf(0,"%s\n",_("File transfer complete."));
+    print_verbose(0,"%s\n",_("File transfer complete."));
   } else {
     errexit("%s\n",ausgabe);
   }
