@@ -40,7 +40,7 @@ phoneentry_to_fileentry (struct pbook_phone_entry** pentries,
     fentries[i]->slot = pentries[i]->slot;
     fentries[i]->number = pentries[i]->number;
     if (uses_unicode) {
-      fentries[i]->text = convert_from_ucs2_hexstring(pentries[i]->text);
+      fentries[i]->text = convert_from_ucs2_hexstring((char*)pentries[i]->text);
     } else {
       fentries[i]->text = convert_from_gsm(pentries[i]->text);
     }
@@ -68,7 +68,7 @@ fileentry_to_phoneentry (struct pbook_entry** fentries,
     pentries[i]->number = fentries[i]->number;
     pentries[i]->numtype = numtype(fentries[i]->number);
     if (use_unicode) {
-      pentries[i]->text = convert_to_ucs2_hexstring(fentries[i]->text);
+      pentries[i]->text = (unsigned char*)convert_to_ucs2_hexstring(fentries[i]->text);
     } else {
       pentries[i]->text = convert_to_gsm(fentries[i]->text);
     }
@@ -98,14 +98,14 @@ void pbook_send_entry (struct parameters* myparams) {
   if (myparams->text != NULL) {
     temp = convert_from_system(myparams->text);
     if (unicode) {
-      pentries[0]->text = convert_to_ucs2_hexstring(temp);
+      pentries[0]->text = (unsigned char*)convert_to_ucs2_hexstring(temp);
     } else {
       pentries[0]->text = convert_to_gsm(temp);
     }
     mem_realloc(temp,0);
   }
   if (pbook_phone_write(myparams->mem,pentries) == 0) {
-    errexit("%s: %s\n",_("Error"),_("something went wrong with sending the phonebook entries"));
+    errexit("%s\n",_("something went wrong with sending the phonebook entries"));
   }
   set_charset("GSM");
 }
@@ -118,14 +118,14 @@ void pbook_send (char* file, char* phonebook) {
   unicode = set_charset("UCS2");
   fentries = pbook_file_read(file);
   if (fentries == NULL) {
-    errexit(_("Error: invalid phonebook file %s.\n"),file);
+    errexit(_("invalid phonebook file %s.\n"),file);
   }
   if (fentries[0] == NULL) {
-    errexit(_("Error: empty phonebook file %s.\n"),file);
+    errexit(_("empty phonebook file %s.\n"),file);
   }
   pentries = fileentry_to_phoneentry(fentries,unicode);
   if (pbook_phone_write(phonebook,pentries) == 0) {
-    errexit("%s: %s\n",_("Error"),_("something went wrong with sending the phonebook entries"));
+    errexit("%s\n",_("something went wrong with sending the phonebook entries"));
   }
   set_charset("GSM");
 }
@@ -145,7 +145,7 @@ void pbook_get (struct parameters* myparams, char* file) {
     pentries = pbook_phone_get(myparams->mem);
   }
   if (pentries == NULL || pentries[0] == NULL) {
-    myprintf(0,"%s\n",_("Nothing to get."));
+    print_verbose(0,"%s\n",_("Nothing to get."));
   } else {
     fentries = phoneentry_to_fileentry(pentries,unicode);
     pbook_file_write(file,fentries);
@@ -156,7 +156,7 @@ void pbook_get (struct parameters* myparams, char* file) {
 void pbook_delete (char* phonebook) {
   char* ausgabe;
   char* temp;
-  int i,min,max;
+  unsigned int i,min=0,max=0;
   enum return_code status;
   struct pbook_phone_entry** pentries = NULL;
   
@@ -167,27 +167,32 @@ void pbook_delete (char* phonebook) {
     ausgabe = at_read_line();
     status = at_line_type(ausgabe,AT_SIE_PB_DELETE,&temp);
     if (status == AT_RET_OK) {
-      myprintf(0,_("%s deleted.\n"),phonebook);
+      print_verbose(0,_("%s deleted.\n"),phonebook);
     } else {
-      errexit("\n%s: %s\n%s\n",_("Error"),temp,_("Possible data corruption!"));
+      errexit("%s\n%s\n",temp,_("Possible data corruption!"));
     }
     mem_realloc(ausgabe,0);
   } else {
-    pbook_get_ranges(phonebook,1,&min,&max,NULL,NULL);
-    pentries = mem_alloc((max-min+2)*sizeof(*pentries),0);
-    for (i = min; i <= max; ++i) {
-      pentries[i-min] = mem_alloc(sizeof(**pentries),0);
-      struct_pbook_phone_entry_init(pentries[i-min]);
-      pentries[i-min]->slot = i;
-    }
-    pentries[max-min+1] = NULL;
-    if (pbook_phone_write(phonebook,pentries) == 0) {
-      errexit("%s: %s\n",_("Error"),_("something went wrong with sending the phonebook entries"));
-    }
+    if (pbook_get_ranges(phonebook,1,&min,&max,NULL,NULL)) {
+      pentries = mem_alloc((max-min+2)*sizeof(*pentries),0);
+      for (i = 0; i <= max-min; ++i) {
+	pentries[i] = mem_alloc(sizeof(**pentries),0);
+	struct_pbook_phone_entry_init(pentries[i]);
+	pentries[i]->slot = i;
+      }
+      pentries[max-min+1] = NULL;
+      if (pbook_phone_write(phonebook,pentries) == 0) {
+	errexit("%s\n",_("something went wrong with sending the phonebook entries"));
+      }
+    } else
+      errexit("%s\n",_("the selected phonebook is not writable"));
   }
 }
 
-int pbook_get_ranges (char* mem, int write, int* min, int* max, int* nrlen, int* txtlen) {
+int pbook_get_ranges (char* mem, int write,
+		      unsigned int* min, unsigned int* max,
+		      int* nrlen, int* txtlen)
+{
   char* command;
   char* ausgabe;
   char* temp;
@@ -261,8 +266,8 @@ int pbook_get_ranges (char* mem, int write, int* min, int* max, int* nrlen, int*
 void pbook_manage (int action, char* outfile,
 		   char** files, struct parameters* myparams)
 {
-  int i = 0;
-  int max;
+  unsigned int i = 0;
+  unsigned int max;
 
   if (myparams->mem == NULL) {
     //SIM myparams->mem seems to be a good default
@@ -285,7 +290,7 @@ void pbook_manage (int action, char* outfile,
     if (pbook_get_ranges(myparams->mem,1,NULL,&max,NULL,NULL))
       myparams->slot = max;
     else
-      errexit("%s: %s\n",_("Error"),_("could not determine last slot."));	  
+      errexit("%s\n",_("could not determine last slot."));	  
   }
 
   switch (action) {
@@ -304,7 +309,7 @@ void pbook_manage (int action, char* outfile,
       if (myparams->slot == SCMXX_SLOT_UNDEFINED) {
 	myparams->slot = pbook_phone_find_empty(myparams->mem,0);
       } else if (myparams->slot < SCMXX_SLOT_PBOOK_MIN) {
-	errexit("%s: %s\n",_("Error"),_("invalid slot defined"));
+	errexit("%s\n",_("invalid slot defined"));
       }
       pbook_send_entry(myparams);
     } else {
