@@ -37,7 +37,7 @@ void sms_manage (int action, char** files,
   struct slot_range range;
   int current_fill;
   struct sms_slot_data** slist;
-  char* pdu;
+  char** pdu;
   int mark_read = 1; //getting SMS with changing status from "unread" to "read"
     
   memlist = NULL;
@@ -149,11 +149,11 @@ void sms_manage (int action, char** files,
       } else {
 	if (myparams->text == NULL &&
 	    (files == NULL || str_len(files[0]) == 0)) {
-	  myprintf(0,"%s\n",_("Hmm, nothing to send?"));
+	  print_verbose(0,"%s\n",_("Hmm, nothing to send?"));
 	}
 	if (myparams->text != NULL) {
 	  pdu = sms_pdu_create(NULL,myparams->text,myparams->number,mysmsopts);
-	  sms_send_pdu(pdu,mysmsopts->direct);
+	  sms_send_pdus(pdu,mysmsopts->direct);
 	  //so we do not submit the same sms multiple times
 	  myparams->text = NULL;
 	}
@@ -161,7 +161,7 @@ void sms_manage (int action, char** files,
 	  i=0;
 	  while (files[i] != NULL) {
 	    pdu = sms_pdu_create(files[i++],NULL,myparams->number,mysmsopts);
-	    sms_send_pdu(pdu,mysmsopts->direct);	
+	    sms_send_pdus(pdu,mysmsopts->direct);	
 	  }
 	}
       }
@@ -193,7 +193,7 @@ void sms_delete_slot (int smsslot) {
     at_gen_sms_delete(smsslot);
     ack = at_read_line();
     if (!strcmp(ack,"OK")) {
-      myprintf(0,_("SMS slot %d was deleted.\n"),smsslot);
+      print_verbose(0,_("SMS slot %d was deleted.\n"),smsslot);
     } else if(!strcmp(ack,"+CMS ERROR: INVALID MEMORY INDEX")) {
       errexit("%s\n",_("This slot is not available."));
     } else if(strstr(ack,"ERROR")!=NULL) {
@@ -219,141 +219,21 @@ void sms_delete_all (struct sms_slot_data** slist) {
   }
 }
 
-void sms_send_slot (int smsslot, char* smsnumber) {
-  char* ausgabe;
-
-  if (str_len(smsnumber) == 0) errexit("%s: %s\n",_("Error"),_("zero-length number"));
-  at_gen_sms_slot_send(smsslot,smsnumber);
-  ausgabe = at_read_line();
-  if (!strcmp(ausgabe,"+CMS ERROR: INVALID MEMORY INDEX")) {
-    errexit("%s\n",_("This slot is not available."));
-  } else if(strstr(ausgabe,"ERROR")!=NULL) {
-    errexit("%s\n",ausgabe);
-  }
-  myprintf(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
-  mem_realloc(ausgabe,0);
-  ausgabe = at_read_line();
-  myprintf(0,"%s: %s\n",_("The phone returned"),ausgabe);
-  mem_realloc(ausgabe,0);
-}
-
-void sms_send_pdu (char* pdu, int direct) {
-  char* command;
-  char* ausgabe;
-  char* ack;
-
-  if (direct) {
-    at_gen_sms_send(sms_tpdu_len(pdu));
-    command = AT_GEN_SMS_SEND;
-  } else {
-    at_gen_sms_store(sms_tpdu_len(pdu));
-    command = AT_GEN_SMS_STORE;
-  }
-  myprintf(0,"%s\n",_("Waiting for data request..."));
-  ack = at_read_line();
-  if (!strncmp(ack,"> ",2)) {
-    mem_realloc(ack,0);
-    myprintf(0,"%s\n",_("Sending data..."));
-    if (tty_write_data(pdu,strlen(pdu)) == -1) {
-      errexit(_("Error on sending data: %s\n"), strerror(errno));
-    }
-    ausgabe = at_read_line();
-    if (strstr(ausgabe,"ERROR")!=NULL) {
-      errexit(_("An error occured on sending the SMS: %s\n"),ausgabe);
-    }
-    if (direct) {
-      myprintf(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
-    } else {
-      myprintf(0,_("The message was saved to SMS memory slot %s\n"),ausgabe+7);
-    }
-    mem_realloc(ausgabe,0);
-    ack = at_read_line();
-    myprintf(0,"%s: %s\n",_("The phone returned"),ack);
-    mem_realloc(ack,0);
-  } else {
-    if (!strncmp(ack,"ERROR",5)) {
-      errexit("%s\n",_("An unknown error occured."));
-    } else if (!strcasecmp(ack,"+CMS ERROR: MEMORY FULL")) {
-      errexit("%s\n",_("There is no slot free to store the sms."));
-    } else {
-      errexit("ERROR: %s\n",ack+12);
-    }
-    mem_realloc(ack,0);
-  }
-}
-
-void sms_send_pdu_all (struct sms_slot_data** slist, char* number, int direct) {
-  int i = 0;
-  
-  if (slist == NULL || *slist == NULL) {
-    return;
-  }
-  while (slist[i] != NULL) {
-    if (slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
-      myprintf(0,_("Trying to send SMS from slot %d...\n"),slist[i]->slot);
-    }
-    if (slist[i]->type != SMS_OUTGOING) {
-      myprintf(0,"%s\n",_("Sending is only possible with SMS of outgoing type."));
-    } else {
-      if (str_len(number) && slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
-	sms_send_slot(slist[i]->slot, number);
-      } else {
-	sms_send_pdu(slist[i]->tpdu,direct);
-      }
-    }
-    ++i;
-  }
-}
-
-char* sms_pdu_create (char* file, char* text, char* number, 
-		      struct smsopts* mysmsopts)
-{
-  char* pdu;
-  char smsinput[(255*155)+1];
-  int myfd;
-  unsigned int i;
-  int eof_present = 0;
-  int read_ret;
+char* sms_pdu_number_lookup (char* number) {
   char* filename;
   struct pbook_entry** cache = NULL;
   struct pbook_entry** found = NULL;
+  unsigned int i;
   unsigned int k = 0;
   ucs4char_t* entry_name;
   char* entry_number = NULL;
-  
-  memset(smsinput,0,sizeof(smsinput));
-
-  if (str_len(text)==0) {// --sms-text was not used, look for other sources
-    myfd=open_myFile_ro(file);
-    for (i=0;i<sizeof(smsinput)-1;i++) {
-      read_ret=read(myfd,&smsinput[i],1);
-      if (read_ret==0) {
-	eof_present=1;
-	break;
-      } else if (read_ret==-1) {
-	errexit("%s\n",_("Error while reading from file."));
-      }
-    }
-    if (eof_present==0) {
-      errexit("%s\n",_("SMS text exceeds buffer size."));
-    }
-    if (str_len(smsinput)==0) {
-      errexit("%s\n",_("No SMS text found (or zero length)."));
-    } else {
-      myprintf(0,_("%s has %d bytes.\n"),file,str_len(smsinput));
-    }
-  } else {
-    if (str_len(text) < sizeof(smsinput)) {
-      strncpy(smsinput,text,sizeof(smsinput)-1);
-    }
-  }
 
   //if number is not a number but text, look it up
   if (is_telephone_number(number) == 0) {
     filename = configfile_get_path (PBOOK_FILE_CACHENAME);
     if (filename == NULL ||
 	file_accessible_ro(filename) == 0) {
-      errexit("%s: %s\n",_("Error"),_("Argument to --number is not a number but phonebook cache is not present."));
+      errexit("%s\n",_("Argument to --number is not a number but phonebook cache is not present."));
     } else {
       cache = pbook_file_read(filename);
       entry_name = convert_from_system(number);
@@ -369,18 +249,18 @@ char* sms_pdu_create (char* file, char* text, char* number,
       }
       mem_realloc(entry_name,0);
       if (found[0] == NULL) {
-	errexit(_("Error: no cache entry for \"%s\" found.\n"),number);
+	errexit(_("no cache entry for \"%s\" found.\n"),number);
       } else if (found[1] != NULL) {
-	fprintf(stderr,_("Error: no unique hit in cache for search term \"%s\":\n"),
-		(number == NULL) ? "" : number);
+	print_error(_("no unique hit in cache for search term \"%s\":\n"),
+		    (number == NULL) ? "" : number);
 	for (k = 0; found[k] != NULL; ++k) {
 	  entry_number = convert_to_system(found[k]->text,REPMODE_QUESTIONMARK);
 	  fprintf(stderr,"%s: %s\n",entry_number,found[k]->number);
 	  entry_number = mem_realloc(entry_number,0);
 	}
-	errexit("%s","");
+	exit(EXIT_FAILURE);
       } else {
-	entry_number = str_dup(found[0]->number);
+	return str_dup(found[0]->number);
       }
       for (k = 0; cache[k] != NULL; ++k) {
 	mem_realloc(cache[k]->number,0);
@@ -391,13 +271,137 @@ char* sms_pdu_create (char* file, char* text, char* number,
       mem_realloc(found,0);
     }
   } else {
-    entry_number = str_dup(number);
+    return str_dup(number);
+  }
+  return NULL;
+}
+
+void sms_send_slot (int smsslot, char* smsnumber) {
+  char* ausgabe;
+  char* entry_number = NULL;
+
+  entry_number = sms_pdu_number_lookup(smsnumber);
+  if (str_len(entry_number) == 0) errexit("%s\n",_("zero-length number"));
+  at_gen_sms_slot_send(smsslot,entry_number);
+  mem_realloc(entry_number,0);
+  ausgabe = at_read_line();
+  if (!strcmp(ausgabe,"+CMS ERROR: INVALID MEMORY INDEX")) {
+    errexit("%s\n",_("This slot is not available."));
+  } else if(strstr(ausgabe,"ERROR")!=NULL) {
+    errexit("%s\n",ausgabe);
+  }
+  print_verbose(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
+  mem_realloc(ausgabe,0);
+  ausgabe = at_read_line();
+  print_verbose(0,"%s: %s\n",_("The phone returned"),ausgabe);
+  mem_realloc(ausgabe,0);
+}
+
+void sms_send_pdus (char** pdu, int direct) {
+  unsigned int i = 0;
+
+  for (; pdu[i] != NULL; ++i)
+    sms_send_pdu(pdu[i],direct);
+}
+
+void sms_send_pdu (char* pdu, int direct) {
+  char* command;
+  char* ausgabe;
+  char* ack;
+
+  if (direct) {
+    at_gen_sms_send(sms_tpdu_len(SMS_OUTGOING,pdu));
+    command = AT_GEN_SMS_SEND;
+  } else {
+    at_gen_sms_store(sms_tpdu_len(SMS_OUTGOING,pdu));
+    command = AT_GEN_SMS_STORE;
+  }
+  print_verbose(0,"%s\n",_("Waiting for data request..."));
+  ack = at_read_line();
+  if (!strncmp(ack,"> ",2)) {
+    mem_realloc(ack,0);
+    print_verbose(0,"%s\n",_("Sending data..."));
+    if (tty_write_data(pdu,strlen(pdu)) == -1) {
+      errexit(_("sending data failed: %s\n"), strerror(errno));
+    }
+    ausgabe = at_read_line();
+    if (strstr(ausgabe,"ERROR")!=NULL) {
+      errexit(_("An error occured on sending the SMS: %s\n"),ausgabe);
+    }
+    if (direct) {
+      print_verbose(0,"%s %s: %s\n",_("The message was sent."),_("Message reference"),ausgabe+7);
+    } else {
+      print_verbose(0,_("The message was saved to SMS memory slot %s\n"),ausgabe+7);
+    }
+    mem_realloc(ausgabe,0);
+    ack = at_read_line();
+    print_verbose(0,"%s: %s\n",_("The phone returned"),ack);
+    mem_realloc(ack,0);
+  } else {
+    if (!strncmp(ack,"ERROR",5)) {
+      errexit("%s\n",_("An unknown error occured."));
+    } else if (!strcasecmp(ack,"+CMS ERROR: MEMORY FULL")) {
+      errexit("%s\n",_("There is no slot free to store the sms."));
+    } else {
+      errexit("%s\n",ack+12);
+    }
+    mem_realloc(ack,0);
+  }
+}
+
+void sms_send_pdu_all (struct sms_slot_data** slist, char* number, int direct) {
+  int i = 0;
+  
+  if (slist == NULL || *slist == NULL) {
+    return;
+  }
+  while (slist[i] != NULL) {
+    if (slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
+      print_verbose(0,_("Trying to send SMS from slot %d...\n"),slist[i]->slot);
+    }
+    if (slist[i]->type != SMS_OUTGOING) {
+      print_verbose(0,"%s\n",_("Sending is only possible with SMS of outgoing type."));
+    } else {
+      if (str_len(number) && slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
+	sms_send_slot(slist[i]->slot, number);
+      } else {
+	sms_send_pdu(slist[i]->tpdu,direct);
+      }
+    }
+    ++i;
+
+  }
+}
+
+char** sms_pdu_create (char* file, char* text, char* number, 
+		       struct smsopts* mysmsopts)
+{
+  char** pdu;
+  char* smsinput;
+  unsigned int smsinputsize;
+  unsigned int i;
+  char* entry_number = NULL;
+  
+  if (str_len(text) == 0) {// --sms-text was not used, look for other sources
+    smsinputsize = file_read_binary(file,&smsinput);
+    if (smsinputsize == 0) {
+      errexit("%s\n",_("No SMS text found (or zero length)."));
+    }
+  } else {
+    smsinput = str_dup(text);
   }
 
-  myprintf(0,"%s\n",_("Creating PDU..."));
+  entry_number = sms_pdu_number_lookup(number);
+  print_verbose(0,"%s\n",_("Creating PDU..."));
   pdu = sms_pdu_create_submit(smsinput,entry_number,mysmsopts);
+  if (pdu == NULL)
+    errexit("%s\n",_("short message encoding failed"));
+
   mem_realloc(entry_number,0);
-  myprintf(1,"PDU: %s\n",pdu);
+  for (i=0; pdu[i] != NULL; ++i)
+    print_verbose(1,"PDU[%d]: %s\n",i,pdu[i]);
+
+  mem_realloc(smsinput,0);
   return pdu;
 }
 
@@ -454,13 +458,13 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
   case AT_RET_OTHER:
     break;
   case AT_RET_ERROR:
-    errexit("\n%s: %s\n",_("Error"),_("unknown cause"));
+    errexit("%s\n",_("unknown cause"));
   case AT_RET_ERROR_CME:
   case AT_RET_ERROR_CMS:
     if (strcasecmp(temp,"INVALID MEMORY INDEX") == 0) {
-      errexit("%s: %s\n",_("Error"),_("This slot is not available."));
+      errexit("%s\n",_("This slot is not available."));
     } else {
-      errexit("%s: %s\n",_("Error"),temp);
+      errexit("%s\n",temp);
     }
     break;
   case AT_RET_OK:
@@ -482,7 +486,7 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
   }
 
   if (slot < SCMXX_SLOT_SMS_MIN) {
-    myprintf(0,"%s\n",_("Looking for SMS of specified type..."));
+    print_verbose(0,"%s\n",_("Looking for SMS of specified type..."));
   }
   do {
     slist[retval]=mem_alloc(sizeof(**slist),0);
@@ -514,11 +518,11 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
       default: break;
       }
       if (mark1 != NULL && mark2 != NULL) {
-	myprintf(0,_("Found %s, %s SMS in slot %d.\n"),
+	print_verbose(0,_("Found %s, %s SMS in slot %d.\n"),
 		 mark1, mark2, slist[retval]->slot);
       } else {
-	myprintf(0,_("Found SMS of unknown type in slot %d.\n"),
-		 slist[retval]->slot)
+	print_verbose(0,_("Found SMS of unknown type in slot %d.\n"),
+		      slist[retval]->slot);
       }
       if (strlen(ausgabe) >= sizeof(slist[retval]->tpdu)) {
 	errexit("%s\n",_("Returned PDU exceeds buffer size."));
@@ -547,15 +551,38 @@ void sms_print (FILE* filefd, char* pipe, struct sms* data) {
   if (str_len(pipe)) { //also send output (splitted) to a given pipe
     pipefd = popen(pipe,"w");
     if (pipefd != NULL) {
-      sms_pdu_print(filefd,data);
+      sms_pdu_print(pipefd,data);
       if (pclose(pipefd) == -1) {
 	fprintf(stderr,"\n");
-	errexit(_("Error on closing pipe \"%s\".\n"),pipe);
+	print_warning(_("closing pipe \"%s\" failed.\n"),pipe);
       }
     }
   }
 }
 
+int strn_reverse_compare (const char* s1, const char* s2, size_t n)
+{
+  size_t s1len = str_len(s1);
+  size_t s2len = str_len(s2);
+
+  if (s1 == NULL && s2 == NULL) return 1;
+  if (s1 == NULL || s2 == NULL) return 0;
+  if (s1len < n) {
+    if(s2len >= n) {
+      return 0;
+    } else { //both are smaller than n
+      if (s1len != s2len) return 0;
+      else n = s1len;
+    }
+  } else {
+    if (s2len < n) return 0;
+    /* else both are longer than n */
+  }
+  
+  if (strcmp(s1+s1len-n,s2+s2len-n) != 0) return 0;
+  else return 1;
+}
+
 void sms_print_all (char* file, char* pipe,
 		    struct sms_slot_data** slist,
 		    char* sort_mode)
@@ -606,7 +633,7 @@ void sms_print_all (char* file, char* pipe,
 
   //merging
   if (sms_multipart_merge(data) < 0) {
-    myprintf(0,"%s: %s\n",_("Warning"),_("Concatenated messages cannot be reassembled."));
+    print_error("%s\n",_("Concatenated messages cannot be reassembled."));
   }
 
   //sorting
@@ -638,14 +665,14 @@ void sms_print_all (char* file, char* pipe,
 	  }
 	  //evaluate if only one match was found
 	  if (found[0] == NULL) {
-	    myprintf(1,_("\nNo cache hit for number \"%s\" with match size %d\n"),
+	    print_verbose(1,_("\nNo cache hit for number \"%s\" with match size %d\n"),
 		     addr->digits,hitlen);
 	  } else if (found[1] == NULL) {
-	    myprintf(1,_("\nCache hit for number \"%s\" with match size %d\n"),
+	    print_verbose(1,_("\nCache hit for number \"%s\" with match size %d\n"),
 		     found[0]->number,hitlen);
 	    addr->text = ucs4dup(found[0]->text);
 	  } else {
-	    myprintf(1,_("\nNo unique hit in cache for number \"%s\" with match size %d\n"),
+	    print_verbose(1,_("\nNo unique hit in cache for number \"%s\" with match size %d\n"),
 		     addr->digits,hitlen);
 	    //try to be more specific on the match
 	    ++hitlen;
@@ -688,7 +715,7 @@ void sms_get_direct (char* file, char* pipe) {
   temp = NULL;
 
   if (str_len(file) == 0 && str_len(pipe) == 0) {
-    errexit("%s: %s\n",_("Error"),_("No output method was specified."));
+    errexit("%s\n",_("No output method was specified."));
   }
 
   at_gen_sms_phase(1);
@@ -698,7 +725,7 @@ void sms_get_direct (char* file, char* pipe) {
   if (status == AT_RET_ANSWER || status == AT_RET_OTHER) {
     mem_realloc(at_read_line(),0);
   } else {
-    errexit("%s: %s\n",_("Error"),_("Could not set Phase 2+ compatible mode."));
+    errexit("%s\n",_("Could not set Phase 2+ compatible mode."));
   }
 
   
@@ -707,7 +734,7 @@ void sms_get_direct (char* file, char* pipe) {
   status = at_line_type(ausgabe,AT_GEN_MSG_DIRECT,NULL);
   mem_realloc(ausgabe,0);
   if (status != AT_RET_OK) {
-    errexit("%s: %s\n",_("Error"),_("Could not set direct mode."));
+    errexit("%s\n",_("Could not set direct mode."));
   }
 
   if (str_len(file) != 0) {
@@ -724,7 +751,7 @@ void sms_get_direct (char* file, char* pipe) {
       mem_realloc(ausgabe,0);
       ausgabe = tty_read_line(at_check_line_end);
       if (ausgabe == NULL) {
-	errexit(_("Error on reading from device: %s\n"), strerror(errno));
+	errexit(_("reading from device failed: %s\n"), strerror(errno));
       }
       if ((temp = index(ausgabe,'\r')) != NULL ||
 	  (temp = index(ausgabe,'\n')) != NULL) {	
@@ -743,10 +770,10 @@ void sms_get_direct (char* file, char* pipe) {
       strncpy(encoded->tpdu,ausgabe,sizeof(encoded->tpdu)-1);
 
       //decode and print the short message
-      //doesn't handle concatenated short messages
+      //does not handle concatenated short messages
       decoded = sms_pdu_decode(encoded->type,encoded);
       if (decoded == NULL || decoded->pdu == NULL) {
-	errexit("%s: %s\n",_("Error"),_("Failed to decode PDU."));
+	errexit("%s\n",_("Failed to decode PDU."));
       }
       struct_sms_init(&data,encoded,decoded);
       sms_print(filefd,pipe,&data);
@@ -765,15 +792,15 @@ void sms_get_direct (char* file, char* pipe) {
     } else if (at_line_type(ausgabe,"+CMTI",NULL) == AT_RET_ANSWER
 	       || at_line_type(ausgabe,"+CDSI",NULL) == AT_RET_ANSWER) {
       //not the message itself but an indication to it (very rare case)
-      myprintf(1,"%s: %s\n",_("Notice"),_("A new message was stored in the phone."));
+      print_notice("%s\n",_("A new message was stored in the phone."));
     } else if (at_line_type(ausgabe,"+CBM",NULL) == AT_RET_ANSWER) {
       mem_realloc(ausgabe,0);
       ausgabe = at_read_line();
-      myprintf(0,"%s:\n%s%s\n\n",_("Notice"),
-	       "A cell broadcast message was received, but decoding is not implemented, yet.\n"
-	       "I would like to implement cell broadcast message decoding but has no examples.\n"
-	       "Please mail this to post@hendrik-sattler.de, including the following PDU:\n",
-	       ausgabe);
+      print_notice("\n%s%s\n\n",
+		   "A cell broadcast message was received, but decoding is not implemented, yet.\n"
+		   "I would like to implement cell broadcast message decoding but has no examples.\n"
+		   "Please mail this to post@hendrik-sattler.de, including the following PDU:\n",
+		   ausgabe);
     }
   }
 }
