@@ -23,10 +23,11 @@
 int main (int argc, char** argv) {
   char* supported_phones[] = {
     "S25",
-    "S35i", "M35i", "MC35i", "C35i",
+    "S35i", "M35i", "MC35i", "C35i", "TC35",
     "SL42", "SLIN", "SL45", "SLIK",
     "C45",  "ME45", "S45",  "S45i",
     "M50",  "MT50", "M50I",
+    "S55",
     //there are even some phones from north america
     "S46",
     NULL
@@ -49,24 +50,27 @@ int main (int argc, char** argv) {
   vendor = NULL;
   model = NULL;
 
-  //setting program's locale
+  //setting locale of this program
   setlocale(LC_ALL,"");
+
+  //init the gettext stuff
   scmxx_gettext_init();
 
-  cfargv = configfile_read("config");
-  if (cfargv != NULL) while(cfargv[cfargc] != NULL) ++cfargc;
-  
-  if (argc <= 1 && cfargc <= 1) {
-    help(argv[0]);
-    exit(EXIT_SUCCESS);
-  }
-
+  //the arguments must be parsed early...
   args_init(&args);
-  if (cfargv != NULL) mem_realloc(args_parse(cfargc,cfargv,&args),0);
+  cfargv = configfile_read("config");
+  if (cfargv != NULL) {
+    while(cfargv[cfargc] != NULL) ++cfargc;
+    mem_realloc(args_parse(cfargc,cfargv,&args),0);
+  }
   files = args_parse(argc,argv,&args);
   use_envvars(&args);
-
   VERBOSE_LEVEL=args.verbose;
+  if (argc <= 1 && cfargc <= 1) args.help = 1;
+
+  //...because we must set the output charset as early as possible
+  //It also configures gettext to the new value.
+  charset_init(args.system_charset,!(args.help||args.version));
 
   if (args.help || args.version) {
     if (args.help) {
@@ -81,27 +85,15 @@ int main (int argc, char** argv) {
   //now open the port
   tty_open(&args.port);
 
-  charset_init(args.system_charset);
   at_init(args.pin);
 
-  if (args.reset) {
-    /* sending a sync frame
-     * the phone may not respond without it because it thinks that all
-     * send commands are still data from a previous run
-     */
-    myprintf(0,"%s...",_("Sending reset frame"));
-    tty_write("\x1a\r\n",3);
-    myprintf(0,"%s\n",_("done"));
-    exit(EXIT_SUCCESS);
-  }
-
   /* Now perform initial phone setup
    */
   //testing the device
-  if (phone_init()) {
-    myprintf(0,"%s\n",_("OK, a modem device is present."));
+  if (phone_init(args.reset)) {
+    print_verbose(0,"%s\n",_("OK, a modem device is present."));
   } else {
-    errexit("%s: %s\n",_("Error"),_("cannot communicate with device."));
+    errexit("%s\n",_("cannot communicate with device."));
   }
   //disabling command echo
   command_echo(0);
@@ -120,58 +112,48 @@ int main (int argc, char** argv) {
     model = get_model();
     if (vendor!=NULL) {
       if (!strcasecmp(vendor,"SIEMENS")) {
-	myprintf(0,_("Detected %s %s\n"),vendor,model);
+	print_verbose(0,_("Detected %s %s\n"),vendor,model);
 	for (i=0; supported_phones[i] != NULL; ++i) {
-	  myprintf(3,_("Comparing to registered model %s\n"),supported_phones[i]);
+	  print_verbose(3,_("Comparing to registered model %s\n"),supported_phones[i]);
 	  if (!strcmp(model,supported_phones[i])) {
 	    break;
 	  }
 	}
 	if (supported_phones[i] == NULL) {
-	  myprintf(1,"%s: %s\n",_("Warning"),
-		   _("this phone was not confirmed to be working with this software."));
+	  print_warning("%s\n",
+			_("this model was not confirmed to be working with this software."));
 	}
-     } else {
-	myprintf(0,"%s: %s\n",_("Warning"),
-		 _("phones from this vendor were not confirmed to be working with this software!"));
+      } else {
+	print_warning("%s\n",
+		      _("phones from this vendor were not confirmed to be working with this software!"));
       }
     }
     mem_realloc(vendor,0);
     mem_realloc(model,0);
   }
 
-  if (args.time || args.smsc || args.dial ||
+  if (args.time || args.smsc || args.dial || args.hangup ||
       args.info || args.meminfo || args.lockinfo ||
       args.lock != NULL) {
-    if (args.time) {
-      set_time();
-    }
-    if (args.smsc) {
-      if (str_len(args.myparams.number)) {
-	set_smsc(args.myparams.number);
-      } else {
-	errexit("%s: %s\n",_("Error"),_("you must define a number with the --number option."));
-      }
-    }
+    if ((args.smsc || args.dial) &&
+	str_len(args.myparams.number) == 0)
+      errexit("%s\n",_("you must define a number with the --number option."));      
+    if (args.time) set_time();
+    if (args.smsc) set_smsc(args.myparams.number);
     if (args.lock != NULL) {
-      lock_toggle(args.lock,args.pin);
+      if (args.state_change > 0) lock_enable(args.lock,args.pin);
+      else if (args.state_change < 0) lock_disable(args.lock,args.pin);
     }
-    if (args.info || args.meminfo || args.lockinfo) {
+    if (args.info || args.meminfo || args.lockinfo)
       info(args.myFILE,args.info,args.meminfo,args.lockinfo);
-    }
-    if (args.dial) {
-      if (str_len(args.myparams.number)) {
-	keypad_dial_number(args.myparams.number);
-      } else {
-	errexit("%s: %s\n",_("Error"),_("you must define a number with the --number option."));
-      }
-    }
+    if (args.dial) dial(args.myparams.number,1);
+    if (args.hangup) hangup();
     exit(EXIT_SUCCESS);
   }
 
   switch(args.scmxx_ftype) {
   default:
-    errexit("%s: %s\n",_("Error"),_("you must specifiy a valid memory type."));
+    errexit("%s\n",_("you must specifiy a valid memory type."));
     break;
   case SCMXX_FTYPE_BINARY:
     file_transfer_manage(args.scmxx_action,files,
