@@ -1,4 +1,3 @@
-
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -17,36 +16,64 @@
 #include "timeincl.h"
 #include "intincl.h"
 
-#define SMS_NUMBER_TYPEMASK  0x70
-#define SMS_NUMTYPE_INTERNAT 0x10
-#define SMS_NUMTYPE_TEXT     0x50  
+/* various SMS options */
+struct smsopts {
+  //for sending
+  char flash;
+  char srr;
+  char unicode;
+  char direct;
+
+  //for printing
+  char* sort_mode;
+};
+
+#define SMS_NUMBER_TYPE_UNKNOWN  0
+#define SMS_NUMBER_TYPE_INTERNAT 1
+#define SMS_NUMBER_TYPE_NAT      2
+#define SMS_NUMBER_TYPE_TEXT     5
+#define SMS_NUMBER_PLAN_UNKNOWN  0
+#define SMS_NUMBER_PLAN_ISDN     1  
+#define SMS_NUMBER_TEXT_LEN   11
+#define SMS_NUMBER_DIGITS_LEN 20
 struct sms_number {
-  unsigned int type; //see ETSI 23.040, Ch. 9.1.2.5
-  unsigned char digits[21];
+  struct { /* see ETSI 23.040, Ch. 9.1.2.5 */
+    /* MSB is always 1! */
+    unsigned int type :3; /* type of number */
+    unsigned int plan :4; /* nummbering plan identification */
+  } toa; /* type of address */
+
+  char digits[SMS_NUMBER_DIGITS_LEN+1]; /* if toa.type != SMS_NUMBER_TYPE_TEXT */
   ucs4char_t* text;
 };
 void struct_sms_number_init (struct sms_number* s);
 void struct_sms_number_delete (struct sms_number* s);
-int struct_sms_number_compare (const struct sms_number* s1, 
-			       const struct sms_number* s2);
+int sms_number_compare (const struct sms_number* s1, 
+			const struct sms_number* s2);
+void sms_number_set (struct sms_number* this, uint8_t toa, const char* number);
+char* sms_number_get (const struct sms_number* this);
+uint8_t sms_number_get_toa (const struct sms_number* this);
 
 struct sms_pdu_time {
   enum {
-    SMS_PDU_TIME_NONE = 0, //ignore value
-    SMS_PDU_TIME_RELATIVE, //value is relative
-    SMS_PDU_TIME_ABSOLUTE  //value is absolute(GMT)
+    SMS_PDU_TIME_NONE = 0, /* ignore value */
+    SMS_PDU_TIME_RELATIVE, /* value is relative */
+    SMS_PDU_TIME_ABSOLUTE  /* value is absolute(GMT) */
   } format;
   time_t value;
 };
 void struct_sms_pdu_time_init (struct sms_pdu_time* s);
+int sms_pdu_time_fill (struct sms_pdu_time* this,
+			const char* format,
+			const char* s, int tz);
+
 
 enum sms_direction {
   SMS_INCOMING,
   SMS_OUTGOING
 };
 
-/* different than the defined values above
- * does not need sms_direction (implied by each value)
+/* sms_direction implied by each value
  */
 enum sms_pdu_type {
   SMS_TYPE_DELIVER,
@@ -56,20 +83,20 @@ enum sms_pdu_type {
   SMS_TYPE_SUBMIT,
   SMS_TYPE_COMMAND
 };
-char* sms_pdu_print_get_type_string (enum sms_pdu_type t);
-
+char* sms_pdu_type_string (enum sms_pdu_type t);
 /* type is the full PDU type field
  * but only the lowest two bits are really needed
  */
-enum sms_pdu_type get_sms_pdu_type (enum sms_direction d, uint8_t type);
+enum sms_pdu_type sms_pdu_type_get (enum sms_direction d, uint8_t type);
+
 
 struct sms_pdu_options {
   enum sms_pdu_type type;
-  uint8_t udh_present; //needed by sms_udh_fill()
-  uint8_t mms; //more messages to send indication
-  uint8_t rd; //reject duplicate request
-  uint8_t rp; //reply path request/indication
-  uint8_t sr; //status report request/indication/qualifier
+  unsigned int udh_present :1; /* needed by sms_udh_fill() */
+  unsigned int mms :1; /* more messages to send indication */
+  unsigned int rd :1; /* reject duplicate request */
+  unsigned int rp :1; /* reply path request/indication */
+  unsigned int sr :1; /* status report request/indication/qualifier */
 };
 void struct_sms_pdu_options_init (struct sms_pdu_options* s);
 
@@ -78,33 +105,35 @@ enum sms_encoding {
   SMS_CHARSET_UCS2,
   SMS_CHARSET_8BIT
 };
+enum sms_pdu_dcs_opt {
+  SMS_DCS_OPT_NONE,
+  SMS_DCS_OPT_CLASS, /* use class */
+  SMS_DCS_OPT_IND    /* use indication */
+};
 struct sms_pdu_dcs {
-  //needed for SMS-STATUS-REPORT (DCS is optional)
-  unsigned char dcs_present;
+  unsigned int dcs_present :1; /* needed for SMS-STATUS-REPORT (DCS is optional) */
 
-  /* note, that not all combination are possible,
-   * see 3GPP TS 23.038, ETSI TS 129 038
-   */
   enum sms_encoding encoding;
-  uint8_t compressed; // =0 not compressed, !=0 compressed
-  uint8_t autodel; // =0 normal mode, !=0 marked as auto-delete
-  enum {
-    SMS_DCS_OPT_NONE,
-    SMS_DCS_OPT_CLASS, //use class
-    SMS_DCS_OPT_IND    //use indication
-  } options;
-  uint8_t class; // 0-3, other values match SMS_DCS_OPT_NONE
-  enum {
-    SMS_DCS_IND_STORE,
-    SMS_DCS_IND_DISCARD
-  } indgroup;
-  uint8_t indsense; // =0 inactive, !=0 active
-  enum {
-    SMS_DCS_IND_VOICE,
-    SMS_DCS_IND_FAX,
-    SMS_DCS_IND_EMAIL,
-    SMS_DCS_IND_OTHER
-  } indtype;
+  unsigned int compressed :1; /* =0 not compressed, !=0 compressed */
+  unsigned int autodel :1; /* =0 normal mode, !=0 marked as auto-delete */
+
+  enum sms_pdu_dcs_opt opttype;
+  union {
+    unsigned int class :2; /* 0-3, other values match SMS_DCS_OPT_NONE */
+    struct {
+      unsigned int sense :1; /* =0 inactive, !=0 active */
+      enum {
+	SMS_DCS_IND_STORE = 0,
+	SMS_DCS_IND_DISCARD
+      } group;
+      enum {
+	SMS_DCS_IND_OTHER = 0,
+	SMS_DCS_IND_VOICE,
+	SMS_DCS_IND_FAX,
+	SMS_DCS_IND_EMAIL
+      } type;
+    } indication;
+  } opt;
 };
 void struct_sms_pdu_dcs_init (struct sms_pdu_dcs* s);
 
@@ -119,16 +148,15 @@ void struct_sms_pdu_message_status_init(struct sms_pdu_message_status* s);
 struct sms_pdu_ud_header {
   uint8_t type;
   uint8_t len;
-  char* data;
+  uint8_t data[137];
 };
 void struct_sms_pdu_ud_header_init (struct sms_pdu_ud_header* s);
-void struct_sms_pdu_ud_header_delete (struct sms_pdu_ud_header* s);
 
 struct sms_pdu_ud {
-  // a NULL terminated list
+  /* a NULL terminated list */
   struct sms_pdu_ud_header** header;
 
-  //the text in the message
+  /* the text in the message */
   ucs4char_t* text;
 
   /* status of the message if a SMS-STATUS-REPORT can be matched
@@ -136,8 +164,10 @@ struct sms_pdu_ud {
    * SMS-STATUS-REPORT matches, this shall be NULL.
    * Never free this pointer from here, except that
    * ack->status_parent is NULL
+   *
+   * never free this from here
    */
-  struct sms_pdu_message_status* ack; //never free this from here
+  struct sms_pdu_message_status* ack;
 };
 void struct_sms_pdu_ud_init (struct sms_pdu_ud* s);
 void struct_sms_pdu_ud_delete (struct sms_pdu_ud* s);
@@ -150,23 +180,23 @@ struct sms_pdu_data {
    * (currently only SMS-SUBMIT and SMS-STATUS-REPORT)
    * and meaning depends on type, too.
    */
-  uint8_t mref; //message reference
+  uint8_t mref;
 
   struct sms_number address;
 
-  uint8_t pid; //protocol ID
+  uint8_t pid; /* protocol ID */
   struct sms_pdu_dcs scheme;
 
   struct sms_pdu_time timedata;
 
-  uint16_t multipart_id; //multipart short message id
-  uint8_t partnum; //number of this part, count from 0
+  uint16_t multipart_id; /* multipart short message id */
+  uint8_t partnum; /* number of this part, count from 0 */
 
   /* ud is a list of size (parts*sizeof(*ud))
    * if parts is 0, ud should be NULL
    */  
-  uint8_t parts; //amount of parts, count from 1
-  struct sms_pdu_ud* ud; //user data,
+  uint8_t parts;
+  struct sms_pdu_ud* ud;
 
   /* service center status of a previously sent message
    * only valid for SMS-STATUS-REPORT
@@ -184,8 +214,8 @@ void struct_sms_tpdu_data_init (struct sms_tpdu_data* s);
 void struct_sms_tpdu_data_delete (struct sms_tpdu_data* s);
 
 struct sms_slot_data {
-  int slot; // >0 for true values
-  char tpdu[363]; //as hexstring
+  int slot; /* >0 for true values */
+  char tpdu[363]; /* as hexstring */
   enum sms_direction type;
 };
 void struct_sms_slot_data_init (struct sms_slot_data* s);
@@ -209,13 +239,13 @@ void struct_sms_delete (struct sms* s);
 /* create a new short message of type SMS-SUBMIT
  * return value is a TPDU allocated with malloc
  */
-#include "options.h"
-char* sms_pdu_create_submit (char* text, char* number, struct smsopts* opts);
+char** sms_pdu_create_submit (char* text, char* number, struct smsopts* opts);
 
 /* both return length of pdu (pdu is part of tpdu)
+ * This is needed as parameter for the at command for sending
  */
-unsigned int sms_tpdu_len (char* tpdu); //for TPDU (means: with SMSC field)
-unsigned int sms_pdu_len (char* pdu); //for PDU
+unsigned int sms_tpdu_len (enum sms_direction d,char* tpdu); //for TPDU (means: with SMSC field)
+unsigned int sms_pdu_len (enum sms_direction d,char* pdu); //for PDU
 
 /***************************
  * decoding short messages *
