@@ -24,10 +24,13 @@
  * size headersize. Headersize also influences the encoding itself!
  * enc_len then contains the value suitable for user data length field
  */
-char* sms_data_encode (enum sms_encoding charset,
-		       ucs4char_t* input,
-		       uint8_t* enc_len,
-		       uint8_t headersize);
+/* char* sms_data_encode (enum sms_encoding charset, */
+/* 		       ucs4char_t* input, */
+/* 		       uint8_t* enc_len, */
+/* 		       uint8_t headersize); */
+char** sms_data_encode (enum sms_encoding charset,
+			struct sms_pdu_ud_header** header,
+			ucs4char_t* input);
 
 /* This calls all functions that are needed to decode the userdata
  * returns NULL on parameter errors else a ucs4char_t* that must be freed
