@@ -16,12 +16,13 @@
 # define _REENTRANT
 #endif
 
-#include <charsets.h>
-#include <helper.h>
 #include <smspdu.h>
-#include <timeincl.h>
 #include "smscoding.h"
 #include "smsudh.h"
+
+#include <charsets.h>
+#include <helper.h>
+#include <timeincl.h>
 #include <gtincl.h>
 
 #include <stdlib.h>
@@ -69,7 +70,7 @@ void sms_statusreport_match (struct sms** list) {
       if (reports[r] != NULL) {
 	rc = reports[r]->decoded->pdu;
 	if (oc->mref == rc->mref &&
-	    struct_sms_number_compare(&oc->address,&rc->address)) {
+	    sms_number_compare(&oc->address,&rc->address)) {
 	  //there may be more than one status report per message...
 	  if (oc->ud[oc->partnum].ack != NULL) {
 	    //...so match the latest one to link with the final status
@@ -126,7 +127,7 @@ int sms_multipart_merge (struct sms** list) {
 	if (c1->options.type == c2->options.type &&
 	    c1->multipart_id == c2->multipart_id &&
 	    c1->parts == c2->parts &&
-	    struct_sms_number_compare(&c1->address,&c2->address)) {
+	    sms_number_compare(&c1->address,&c2->address)) {
 	  if (c1->partnum == c2->partnum ||
 	      (multipart[m]->encoded[c1->partnum] != NULL)) {
 	    /* We now have two concatenated short messages
@@ -195,7 +196,7 @@ int sms_multipart_merge (struct sms** list) {
 }
 
 
-enum sms_pdu_type get_sms_pdu_type (enum sms_direction d, uint8_t type) {
+enum sms_pdu_type sms_pdu_type_get (enum sms_direction d, uint8_t type) {
   switch (d) {
   case SMS_INCOMING:
     switch (type&3) {
@@ -216,12 +217,13 @@ enum sms_pdu_type get_sms_pdu_type (enum sms_direction d, uint8_t type) {
   return SMS_TYPE_DELIVER;
 }
 
+static
 void sms_pdu_decode_options (enum sms_direction d, uint8_t type,
-			  struct sms_pdu_options* p)
+			     struct sms_pdu_options* p)
 {
   if (p == NULL) return;
 
-  p->type = get_sms_pdu_type(d,type);
+  p->type = sms_pdu_type_get(d,type);
   if (type&0x04) {
     if (p->type == SMS_TYPE_DELIVER ||
 	p->type == SMS_TYPE_STATUS_REPORT) {
@@ -247,6 +249,7 @@ void sms_pdu_decode_options (enum sms_direction d, uint8_t type,
   }
 }
 
+static
 void sms_pdu_dcs_decode (uint8_t dcs, struct sms_pdu_dcs* p) {
   if (p == NULL) return;
   p->dcs_present = 1;
@@ -267,31 +270,31 @@ void sms_pdu_dcs_decode (uint8_t dcs, struct sms_pdu_dcs* p) {
   if ((dcs&0x80) == 0) {
     if (dcs&0x40) p->autodel = 1;
     if (dcs&0x20) p->compressed = 1;
-    if (dcs&0x08) {
-      p->options = SMS_DCS_OPT_CLASS;
-      p->class = dcs&3;
+    if (dcs&0x10) {
+      p->opttype = SMS_DCS_OPT_CLASS;
+      p->opt.class = dcs&3;
     }
   } else {
     if (dcs >= 0xF0) {
-      p->options = SMS_DCS_OPT_CLASS;
-      p->class = dcs&3;
+      p->opttype = SMS_DCS_OPT_CLASS;
+      p->opt.class = dcs&3;
     } else if (dcs >= 0xC0) {
-      p->options = SMS_DCS_OPT_IND;
+      p->opttype = SMS_DCS_OPT_IND;
       if ((dcs&0x30) == 0) {
-	p->indgroup = SMS_DCS_IND_DISCARD;
+	p->opt.indication.group = SMS_DCS_IND_DISCARD;
       } else {
-	p->indgroup = SMS_DCS_IND_STORE;
+	p->opt.indication.group = SMS_DCS_IND_STORE;
       }
       if (dcs&0x08) {
-	p->indsense = 1;
+	p->opt.indication.sense = 1;
       } else {
-	p->indsense = 0;
+	p->opt.indication.sense = 0;
       }
       switch(dcs&0x03) {
-      case 0: p->indtype = SMS_DCS_IND_VOICE; break;
-      case 1: p->indtype = SMS_DCS_IND_FAX; break;
-      case 2: p->indtype = SMS_DCS_IND_EMAIL; break;
-      case 3: p->indtype = SMS_DCS_IND_OTHER; break;
+      case 0: p->opt.indication.type = SMS_DCS_IND_VOICE; break;
+      case 1: p->opt.indication.type = SMS_DCS_IND_FAX; break;
+      case 2: p->opt.indication.type = SMS_DCS_IND_EMAIL; break;
+      case 3: p->opt.indication.type = SMS_DCS_IND_OTHER; break;
       }
     }
   }
@@ -299,6 +302,7 @@ void sms_pdu_dcs_decode (uint8_t dcs, struct sms_pdu_dcs* p) {
 
 /* returns the number of character to advance in pdu
  */
+static
 unsigned int sms_pdu_address_decode (char* pdu, struct sms_number* n)
 {
   unsigned int len;
@@ -307,10 +311,10 @@ unsigned int sms_pdu_address_decode (char* pdu, struct sms_number* n)
   len = hexstr2int(pdu,2);
   len += len%2;
   struct_sms_number_init(n);
-  n->type = hexstr2int(pdu+2,2);
+  sms_number_set(n,hexstr2int(pdu+2,2),NULL);
   if (len) {
-    switch (n->type&SMS_NUMBER_TYPEMASK) {
-    case SMS_NUMTYPE_TEXT:
+    switch (n->toa.type) {
+    case SMS_NUMBER_TYPE_TEXT:
       //those are octets, not semi-octets
       n->text = sms_data_decode(SMS_CHARSET_GSM,
 				pdu+4,(len*4)/7,0);
@@ -328,59 +332,31 @@ unsigned int sms_pdu_address_decode (char* pdu, struct sms_number* n)
   return 2+2+len;
 }
 
+static
 unsigned int sms_pdu_scts_decode (char* pdu, struct sms_pdu_time* t)
 {
   char* temp;
-  struct tm sct_tm;
+  int tz;
 
-  t->format = SMS_PDU_TIME_ABSOLUTE;
-  temp = mem_alloc(18,1);
+  //read real timezone offset
+  temp = mem_alloc(3,0);
+  temp[0] = pdu[13];
+  temp[1] = pdu[12];
+  tz = atoi(temp);
+  /* parse negation bit */
+  if (tz >= 80) tz = (-1)*(tz-80);
+
+  temp = mem_realloc(temp,18);
+  memset(temp,0,18);
   sprintf(temp,"%c%c/%c%c/%c%c,%c%c:%c%c:%c%c",
 	  pdu[1],pdu[0],pdu[3],pdu[2],pdu[5],pdu[4],
 	  pdu[7],pdu[6],pdu[9],pdu[8],pdu[11],pdu[10]);
-  memset(&sct_tm,0,sizeof(sct_tm));
-  if (strptime(temp,"%y/%m/%d,%H:%M:%S",&sct_tm) != NULL) {
-    /* Not set by strptime(); tells mktime() to determine whether
-     * daylight saving time is in effect
-     */
-    sct_tm.tm_isdst = -1;
-    /* work-around for lazy *BSD implementation of strptime()
-     * that does not set tm_yday in struct tm
-     */
-    tzset();
-    t->value = mktime(&sct_tm);
-    localtime_r(&t->value,&sct_tm);
-    
-    /* mktime() does not work correctly for this, so we use the following from
-     * http://www.opengroup.org/onlinepubs/007904975/basedefs/xbd_chap04.html#tag_04_14
-     * (slightly modified)
-     */
-    t->value = sct_tm.tm_sec + sct_tm.tm_min*60 + sct_tm.tm_hour*3600;
-    t->value += 86400 * (sct_tm.tm_yday
-			 + (sct_tm.tm_year-70)*365
-			 + (sct_tm.tm_year-69)/4
-			 - (sct_tm.tm_year-1)/100
-			 + (sct_tm.tm_year+299)/400);
-    
-    //read real timezone offset
-    temp = mem_realloc(temp,3);
-    memset(temp,0,3);
-    temp[0] = pdu[13];
-    temp[1] = pdu[12];
-    if ((atoi(temp)%80)/4 <= 12) {
-      if (atoi(temp) >= 80) { //apply timezone offset
-	t->value += (atoi(temp)%80)*15*60;
-      } else {
-	t->value -= (atoi(temp)%80)*15*60;
-      }
-    }
-  } else { //fallback for strptime failure
-    t->format = SMS_PDU_TIME_NONE;
-  }
+  sms_pdu_time_fill(t,"%y/%m/%d,%H:%M:%S",temp,tz);
   mem_realloc(temp,0);
   return 14;
 }
 
+static
 void sms_pdu_userdata_decode (char* pdu, struct sms_pdu_data* sms)
 {
   uint8_t len = hexstr2int(pdu,2);
@@ -405,6 +381,7 @@ void sms_pdu_userdata_decode (char* pdu, struct sms_pdu_data* sms)
   }
 }
 
+static
 struct sms_pdu_data* sms_pdu_decode_statusreport (char* pdu) {
   struct sms_pdu_data* sms;
   uint8_t temp = 0;
@@ -462,6 +439,7 @@ struct sms_pdu_data* sms_pdu_decode_statusreport (char* pdu) {
   return sms;
 }
 
+static
 struct sms_pdu_data* sms_pdu_decode_deliver (char* pdu)
 {
   struct sms_pdu_data* sms;
@@ -487,6 +465,7 @@ struct sms_pdu_data* sms_pdu_decode_deliver (char* pdu)
   return sms;
 }
 
+static
 struct sms_pdu_data* sms_pdu_decode_submit (char* pdu)
 {
   struct sms_pdu_data* sms;
@@ -598,10 +577,9 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   pdu = NULL;
 
   //extracting SMSC
-  sca_len = hexstr2int(sms->tpdu,2);
+  sca_len = hexstr2int(sms->tpdu,2)*2;
   if (sca_len > 0) {
-    sca_len *= 2;
-    decoded->sca.type = hexstr2int(sms->tpdu+2,2);
+    sms_number_set(&(decoded->sca),hexstr2int(sms->tpdu+2,2),NULL);
     for (i=0;i<sca_len-2;i+=2) {
       decoded->sca.digits[i] = sms->tpdu[4+i+1];
       decoded->sca.digits[i+1] = sms->tpdu[4+i];
@@ -613,7 +591,7 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   pdu = str_dup(sms->tpdu+sca_len+2);
 
   //checking if PDUtype is supported and can thus be decoded
-  switch (get_sms_pdu_type(type,hexstr2int(pdu,2))) {
+  switch (sms_pdu_type_get(type,hexstr2int(pdu,2))) {
   case SMS_TYPE_DELIVER:
     decoded->pdu = sms_pdu_decode_deliver(pdu);
     break;
@@ -627,7 +605,7 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   case SMS_TYPE_DELIVER_REPORT:
   case SMS_TYPE_COMMAND:
     fprintf(stderr,"%s: %s\n",_("Unsupported pdu type"),
-            sms_pdu_print_get_type_string(get_sms_pdu_type(type,hexstr2int(pdu,2))));
+            sms_pdu_type_string(sms_pdu_type_get(type,hexstr2int(pdu,2))));
     struct_sms_tpdu_data_delete(decoded);
     decoded = mem_realloc(decoded,0);
     break;
@@ -635,3 +613,128 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   mem_realloc(pdu,0);
   return decoded;
 }
+
+static
+unsigned int sms_pdu_submitdeliver_len (enum sms_pdu_type type,
+					uint8_t vpsize,
+					char* pdu)
+{
+  struct sms_pdu_dcs d;
+  size_t retval = 1;
+  uint8_t temp;
+
+  if (type == SMS_TYPE_SUBMIT) {
+    /* minimal PDU length 7 octets */
+    if (strlen(pdu) < 7*2) return 0; //invalid
+    retval += 1; //MR
+  } else {
+    /* minimal PDU length 13 octets */
+    if (strlen(pdu) < 13*2) return 0; //invalid
+  }
+
+  temp = hexstr2int(pdu+2*retval,2);
+  retval += 2 + (temp + temp%2)/2; //DA
+
+  retval += 1; //PID
+
+  if (strlen(pdu) < (retval+1)*2) return 0;
+  temp = hexstr2int(pdu+2*retval,2);
+  sms_pdu_dcs_decode(temp,&d);
+  retval += 1+vpsize; //DCS,VP
+
+  if (strlen(pdu) < (retval+1)*2) return 0;
+  temp = hexstr2int(pdu+2*retval,2);
+  retval += 1;
+  if (d.encoding == SMS_CHARSET_GSM) {
+    if (temp > 160) return 0;
+    retval += (temp*7)/8;
+    if (temp%8) ++retval;
+  } else {
+    if (temp > 140) return 0;
+    retval += temp;
+  }
+
+  return retval;  
+}
+
+static
+unsigned int sms_pdu_statusreport_len (char* pdu) {
+  struct sms_pdu_dcs d;
+  size_t retval = 1; //MR
+  uint8_t temp;
+
+  /* minimal PDU length 20 octets */
+  if (strlen(pdu) < 20*2) return 0; //invalid
+
+  temp = hexstr2int(pdu+2*retval,2);
+  retval += 2 + (temp + temp%2)/2; //RA
+
+  retval += 7 + 7 + 1; //SCTS,DT,ST
+
+  if (strlen(pdu) < (retval+1)*2) return 0;
+  temp = hexstr2int(pdu+2*retval,2);
+  if (temp&0x01) ++retval;
+  temp = 0;
+  if (temp&0x02) {
+    temp = hexstr2int(pdu+2*retval,2);
+    ++retval;
+  }
+  sms_pdu_dcs_decode(temp,&d);
+  if (temp&0x04) {
+    if (strlen(pdu) < (retval+1)*2) return 0;
+    temp = hexstr2int(pdu+2*retval,2);
+    if (d.encoding == SMS_CHARSET_GSM) {
+      retval += (temp*7)/8;
+      if (temp%8) ++retval;
+    } else {
+      retval += temp;
+    }
+  }
+  if (retval > 164) return 0;
+
+  return retval;  
+}
+
+unsigned int sms_pdu_len (enum sms_direction d,
+			  char* pdu)
+{
+  uint8_t type;
+  struct sms_pdu_options p;
+  unsigned int temp = 0;
+
+  if (str_len(pdu) < 2) return 0;
+  type = hexstr2int(pdu,2);
+  sms_pdu_decode_options(d,type,&p);
+ 
+
+  switch (p.type) {
+  case SMS_TYPE_SUBMIT:
+    switch (type&0x18) { //VP
+    default: temp += 6;
+    case 0x10: temp += 1;
+    case 0x00: break;
+    }
+    return sms_pdu_submitdeliver_len(p.type,temp,pdu);
+  case SMS_TYPE_DELIVER:
+    return sms_pdu_submitdeliver_len(p.type,7,pdu);
+  case SMS_TYPE_STATUS_REPORT:
+    return sms_pdu_statusreport_len(pdu);
+  default:
+    //short but does not look for invalid PDUs
+    temp = str_len(pdu)/2;
+    if (temp > 164) return 0;
+    else return temp;
+  }
+}
+
+unsigned int sms_tpdu_len (enum sms_direction d,
+			   char* tpdu)
+{
+  unsigned int temp = 0;
+
+  if (str_len(tpdu) < 2) return 0;
+  temp = hexstr2int(tpdu,2);
+  if (temp > 11) return 0;
+  return sms_pdu_len(d,tpdu+2*(temp+1));
+}
+
