@@ -16,29 +16,22 @@
 # define _REENTRANT
 #endif
 
-#include <common.h>
-#include <charsets.h>
-#include <helper.h>
 #include <smspdu.h>
 #include "smscoding.h"
 #include "smsudh.h"
+
+#include <charsets.h>
+#include <helper.h>
 #include <gtincl.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
-unsigned int sms_pdu_len (char* pdu) {
-  //short but does not look for invalid PDUs
-  return str_len(pdu)/2;
-}
-
-unsigned int sms_tpdu_len (char* tpdu) {
-  return sms_pdu_len(tpdu+2+hexstr2int(tpdu,2));
-}
-
-char* sms_pdu_create_submit (char* text, char* number,
-			     struct smsopts* opts)
+static
+char* sms_pdu_create_submit_frame (char* number,
+				   struct smsopts* opts,
+				   int header_present)
 {
   uint8_t sca_len = 0;
   struct sms_number sca; //short message service center address
@@ -50,19 +43,17 @@ char* sms_pdu_create_submit (char* text, char* number,
   uint8_t dcs; //data coding scheme
   uint8_t vp_rel = 0xff; //validity period (0xff = max = 63 weeks)
   //char vp_abs[15]; //validity period (YYMMDDhhmmssTZ)
-  uint8_t udl; //user data length
-  unsigned char* ud; //user data
 
-  ucs4char_t* wide_str;
   unsigned int i;
-
-  char* pdu;
+  unsigned int k;
+  char* frame = mem_alloc(176*2+1,1);
+  char* temp;
 
   /* not yet supported
    * zero length means that the phone will use its default
    */
   struct_sms_number_init(&sca);
-  sca.type = numtype(sca.digits);
+  sms_number_set(&sca,numtype(sca.digits),NULL);
   sca_len = strlen(sca.digits)/2;
   if (sca_len) {
     ++sca_len;
@@ -75,6 +66,9 @@ char* sms_pdu_create_submit (char* text, char* number,
   if (opts->srr) {
     pdutype |= (1<<5); //status report request;
   }
+  if (header_present) {
+    pdutype |= (1<<6); //user data header indication
+  }
 
   /* message reference must be zero for Siemens phones!
    */
@@ -82,22 +76,18 @@ char* sms_pdu_create_submit (char* text, char* number,
 
   /* destination number
    */
-  struct_sms_number_init(&da);
-  da.type = numtype(number);
   if (is_telephone_number(number) == 0) {
     fprintf(stderr,"%s: %s\n",_("Error"),_("No valid number specified."));
-    exit(EXIT_FAILURE);
+    return NULL;
   }
-  if ((da.type&SMS_NUMBER_TYPEMASK) == SMS_NUMTYPE_INTERNAT) {
-    i = 1;
-  } else {
-    i = 0;
+  struct_sms_number_init(&da);
+  sms_number_set(&da,numtype(number),number);
+  temp = sms_number_get(&da);
+  if (temp == NULL || strcmp(temp,number) != 0) {
+    fprintf(stderr,_("%s: sms number cannot have more than %d digits.\n"),_("Error"),sizeof(da.digits)-1);
+    return NULL;
   }
-  if (strlen(number+i) > sizeof(da.digits)-1) {
-    fprintf(stderr,_("Error: sms number cannot have more than %d digits.\n"),sizeof(da.digits)-1);
-    exit(EXIT_FAILURE);
-  }
-  strncpy(da.digits,number+i,sizeof(da.digits)-1);
+  mem_realloc(temp,0);
   da_len = strlen(da.digits);
 
   /* support "flash-SMS" and unicode here
@@ -114,44 +104,53 @@ char* sms_pdu_create_submit (char* text, char* number,
 
   //leave pid and vp_rel as is
 
-  /* proces user data
-   */
-  wide_str = convert_from_system(text);
-  ud = sms_data_encode((opts->unicode) ? SMS_CHARSET_UCS2 : SMS_CHARSET_GSM,
-		       wide_str,&udl,0);
-  mem_realloc(wide_str,0);
-  if (ud == NULL) {
-    fprintf(stderr,_("SMS text is too long (max. %d characters).\n"),
-	    (opts->unicode) ? 140/2 : 140*8/7);
-    if (!opts->unicode) {
-      fprintf(stderr,"%s\n",
-	      _("Please be aware that some characters "
-	        "are encoded as 14bit (instead of 7bit), "
-	        "e.g. the euro character."));
-    }
-    exit(EXIT_FAILURE);
-  }
-
-  //create PDU
-  pdu = mem_alloc((176*2)+1,1);
-  sprintf(pdu,"%02X",sca_len);
+  //create PDU frame
+  sprintf(frame,"%02X",sca_len);
   if (sca_len) {
-    sprintf(pdu+2,"%02X%s",sca.type,sca.digits);
+    sprintf(frame+2,"%02X%s",sms_number_get_toa(&sca),sca.digits);
   }
-  sprintf(pdu+strlen(pdu),"%02X%02X%02X%02X",
-	  pdutype,mr,da_len,da.type);
-  for (i = 0; i+2 < strlen(da.digits); i += 2) {
-    sprintf(pdu+strlen(pdu),"%c%c",
-	    da.digits[i+1],da.digits[i]);
+  sprintf(frame+strlen(frame),"%02X%02X%02X%02X",
+	  pdutype,mr,da_len,sms_number_get_toa(&da));
+  k = strlen(da.digits);
+  for (i = 0; i < k; i += 2) {
+    sprintf(frame+strlen(frame),"%c%c",
+	    (i+1>=k && k%2) ? 'F' : da.digits[i+1],
+	    da.digits[i]);
   }
-  sprintf(pdu+strlen(pdu),"%c%c",
-	  ((strlen(da.digits)%2) ? 'F' : da.digits[i+1]),
-	  da.digits[i]);
-  sprintf(pdu+strlen(pdu),"%02X%02X%02X%02X%s",
-	  pid,dcs,vp_rel,udl,ud);
-  mem_realloc(ud,0);
+  sprintf(frame+strlen(frame),"%02X%02X%02X",
+	  pid,dcs,vp_rel);
+
   struct_sms_number_delete(&sca);
   struct_sms_number_delete(&da);
-  return pdu;
+
+  return frame;
 }
 
+char** sms_pdu_create_submit (char* text, char* number,
+			      struct smsopts* opts)
+{
+  ucs4char_t* wide_str = convert_from_system(text);
+  char** pdu = sms_data_encode((opts->unicode) ? SMS_CHARSET_UCS2 : SMS_CHARSET_GSM,
+			      NULL, wide_str);
+  unsigned int i = 0;
+  unsigned int k;
+  char* frame;
+
+  mem_realloc(wide_str,0);
+
+  if (pdu == NULL) return NULL;
+
+  frame = sms_pdu_create_submit_frame(number,opts,
+				      (pdu[1] == NULL)? 0 : 1);
+  if (frame == NULL) return NULL;
+
+  k = strlen(frame);
+  for (; pdu[i] != NULL; ++i) {
+    pdu[i] = mem_realloc(pdu[i],(176*2)+1);
+    memmove(pdu[i]+k,pdu[i],strlen(pdu[i])+1);
+    memcpy(pdu[i],frame,k);
+  }
+  mem_realloc(frame,0);
+
+  return pdu;
+}
