@@ -16,18 +16,20 @@
 # define _REENTRANT
 #endif
 
-#include <charsets.h>
-#include <helper.h>
 #include <smspdu.h>
-#include <timeincl.h>
 #include "smscoding.h"
 #include "smsudh.h"
+
+#include <charsets.h>
+#include <helper.h>
+#include <timeincl.h>
 #include <gtincl.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
+static
 void sms_pdu_print_item_slot (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
   unsigned int i = 0;
@@ -43,6 +45,7 @@ void sms_pdu_print_item_slot (FILE* fp, struct sms* sms) {
   }
 }
 
+static
 void sms_pdu_print_item_address (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
   char* temp;
@@ -56,21 +59,22 @@ void sms_pdu_print_item_address (FILE* fp, struct sms* sms) {
     case SMS_TYPE_DELIVER: temp =  _("From"); break;
     case SMS_TYPE_STATUS_REPORT: temp = _("Original To"); break;
     }
-    fprintf(fp,"%s: ",temp);
-    if (str_len(pdu->address.digits) > 0) {
-      fprintf(fp,"%s%s ",
-	      (((pdu->address.type&SMS_NUMBER_TYPEMASK) == SMS_NUMTYPE_INTERNAT) ? "+" : ""),
-	      pdu->address.digits);
+    fprintf(fp,"%s:",temp);
+    if (str_len(temp) > 0) {
+      temp = sms_number_get(&pdu->address);
+      if (temp != NULL) fprintf(fp," %s",temp);
+      mem_realloc(temp,0);
     }
     if (ucs4len(pdu->address.text)) {
       temp = convert_to_system(pdu->address.text,REPMODE_QUESTIONMARK);
-      fprintf(fp,"(%s)",temp);
+      fprintf(fp," (%s)",temp);
       mem_realloc(temp,0);      
     }
     fprintf(fp,"\n");
   }
 }
 
+static
 void sms_pdu_print_item_time (FILE* fp, enum sms_pdu_type type,
 			      struct sms_pdu_time* t)
 {
@@ -121,18 +125,15 @@ void sms_pdu_print_item_time (FILE* fp, enum sms_pdu_type type,
   }
 }
 
+static
 void sms_pdu_print_item_smsc (FILE* fp, struct sms* sms) {
-  struct sms_pdu_data* pdu;
-
-  pdu = sms->decoded->pdu;
-  if (strlen(sms->decoded->sca.digits)) {
-    fprintf(fp,"%s: %s%s\n",_("SMSC number"),
-	    (((sms->decoded->sca.type&SMS_NUMBER_TYPEMASK) == SMS_NUMTYPE_INTERNAT) ? "+" : ""),
-	    sms->decoded->sca.digits);
-  }
+  char* temp = sms_number_get(&(sms->decoded->sca));
+  if (str_len(temp) > 0)
+    fprintf(fp,"%s: %s\n",_("SMSC number"),temp);
+  mem_realloc(temp,0);
 }
 
-char* sms_pdu_print_get_type_string (enum sms_pdu_type t) {
+char* sms_pdu_type_string (enum sms_pdu_type t) {
   switch (t) {
   case SMS_TYPE_DELIVER: return "SMS-DELIVER";
   case SMS_TYPE_SUBMIT: return "SMS-SUBMIT";
@@ -144,11 +145,12 @@ char* sms_pdu_print_get_type_string (enum sms_pdu_type t) {
   return "";
 }
 
+static
 void sms_pdu_print_item_options (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
 
   pdu = sms->decoded->pdu;
-  fprintf(fp,"%s: %s\n",_("PDU type"),sms_pdu_print_get_type_string(pdu->options.type));
+  fprintf(fp,"%s: %s\n",_("PDU type"),sms_pdu_type_string(pdu->options.type));
   if (pdu->options.mms || pdu->options.rd ||
       pdu->options.rp || pdu->options.sr) {
     fprintf(fp,"%s:",_("PDU flags"));
@@ -160,6 +162,7 @@ void sms_pdu_print_item_options (FILE* fp, struct sms* sms) {
   }
 }
 
+static
 void sms_pdu_print_item_pid (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
 
@@ -201,6 +204,7 @@ void sms_pdu_print_item_pid (FILE* fp, struct sms* sms) {
   }
 }
 
+static
 void sms_pdu_print_item_dcs (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
   char* temp;
@@ -213,20 +217,20 @@ void sms_pdu_print_item_dcs (FILE* fp, struct sms* sms) {
   case SMS_CHARSET_UCS2: fprintf(fp," %s","UCS-2"); break;
   case SMS_CHARSET_8BIT: fprintf(fp," %s","8bit"); break;
   }
-  if (pdu->scheme.options == SMS_DCS_OPT_CLASS &&
-      pdu->scheme.class <= 3) {
-    fprintf(fp,_(" (class %d)"),pdu->scheme.class);
+  if (pdu->scheme.opttype == SMS_DCS_OPT_CLASS &&
+      pdu->scheme.opt.class <= 3) {
+    fprintf(fp,_(" (class %d)"),pdu->scheme.opt.class);
   }
   if (pdu->scheme.autodel) fprintf(fp," %s",_("marked as auto-delete"));
-  if (pdu->scheme.options == SMS_DCS_OPT_IND) {
-    switch(pdu->scheme.indtype) {
+  if (pdu->scheme.opttype == SMS_DCS_OPT_IND) {
+    switch(pdu->scheme.opt.indication.type) {
     case SMS_DCS_IND_VOICE: temp = _("voicemail"); break;
     case SMS_DCS_IND_FAX: temp = _("fax"); break;
     case SMS_DCS_IND_EMAIL: temp = _("e-mail"); break;
     case SMS_DCS_IND_OTHER: temp = _("misc."); break;
     }
     fprintf(fp,"\n%s:",_("Indication"));
-    if (pdu->scheme.indsense) {
+    if (pdu->scheme.opt.indication.sense) {
       fprintf(fp," %s\n",_("new message(s) waiting"));
     } else {
       fprintf(fp," %s\n",_("no more messages waiting"));
@@ -235,6 +239,7 @@ void sms_pdu_print_item_dcs (FILE* fp, struct sms* sms) {
   fprintf(fp,"\n");
 }
 
+static
 void sms_pdu_print_item_text (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
   uint16_t textlen = 0;
@@ -261,6 +266,7 @@ void sms_pdu_print_item_text (FILE* fp, struct sms* sms) {
   fprintf(fp,"\n");
 }
 
+static
 void sms_pdu_print_item_status (FILE* fp, struct sms_pdu_message_status* m) {
   char* temp;
   char* temp2;
@@ -309,6 +315,7 @@ void sms_pdu_print_item_status (FILE* fp, struct sms_pdu_message_status* m) {
   sms_pdu_print_item_time(fp,SMS_TYPE_STATUS_REPORT,&m->t);
 }
 
+static
 void sms_pdu_print_item_mr (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
 
@@ -325,6 +332,7 @@ void sms_pdu_print_item_mr (FILE* fp, struct sms* sms) {
   }
 }
 
+static
 void sms_pdu_print_deliver (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
 
@@ -347,6 +355,7 @@ void sms_pdu_print_deliver (FILE* fp, struct sms* sms) {
   fprintf(fp,"\n");
 }
 
+static
 void sms_pdu_print_statusreport (FILE* fp, struct sms* sms) {
   struct sms_pdu_data* pdu;
 
@@ -391,7 +400,7 @@ void sms_pdu_print (FILE* fp, struct sms* sms) {
   case SMS_TYPE_DELIVER_REPORT:
   case SMS_TYPE_COMMAND:
     fprintf(stderr,"%s: %s\n\n",_("Unsupported pdu type"),
-	    sms_pdu_print_get_type_string(sms->decoded->pdu->options.type));
+	    sms_pdu_type_string(sms->decoded->pdu->options.type));
     break;
   }
 }
