@@ -10,15 +10,247 @@
  ***************************************************************************/
 
 //own headers
+#include "smsudh.h"
+
 #include <helper.h>
 #include <intincl.h>
-#include "smsudh.h"
 #include <gtincl.h>
 
 //standard headers
 #include <stdio.h>
 
-struct sms_pdu_ud_header_def header_defs[] = {
+struct sms_pdu_ud_header*
+sms_udh_multipart16_get (uint16_t seqnum,
+			 uint8_t index,
+			 uint8_t total)
+{
+  struct sms_pdu_ud_header* retval;
+  
+  retval = mem_alloc(sizeof(*retval),0);
+  struct_sms_pdu_ud_header_init(retval);
+  retval->type = 0x08;
+  retval->len = 4;
+  retval->data[0] = (seqnum >>8)&0xFF;
+  retval->data[1] = seqnum&0xFF;
+  retval->data[2] = total;
+  retval->data[3] = index;
+  return retval;
+}
+
+static
+void sms_udh_indication_print (FILE* fp, uint8_t* data) {
+  char* type = NULL;
+
+  fprintf(fp,"%s:",_("Indication"));
+  switch (data[0]&0x4F) {
+  case 0: type = _("voicemail"); break;
+  case 1: type = _("fax"); break;
+  case 2: type = _("e-mail"); break;
+  case 3: type = _("misc."); break;
+  }
+  fprintf(fp,ngettext("%d %s message waiting","%d %s messages waiting",data[1]),data[1],type);
+  
+}
+
+static
+void sms_udh_wcmpdu_print (FILE* fp, uint8_t* data) {
+  data = data;
+  fprintf(fp,"%s\n",_("Wireless control message protocol data unit"));
+}
+
+static
+void sms_udh_smstk_print (FILE* fp, uint8_t* data) {
+  data = data;
+  fprintf(fp,_("SIM toolkit security header"));
+}
+
+static
+void sms_udh_multipart8_print (FILE* fp, uint8_t* data) {
+  fprintf(fp,_("Multipart message: sequence number = %d, part %d of %d"),
+	  data[0],data[2],data[1]);
+}
+
+static
+void sms_udh_multipart16_print (FILE* fp, uint8_t* data) {
+  fprintf(fp,_("Multipart message: sequence number = %d, part %d of %d"),
+	  (data[0]<<8)+data[1],data[3],data[2]);
+}
+
+static
+void sms_udh_application8_print (FILE* fp, uint8_t* data) {
+  fprintf(fp,_("Application port: destination port = %d, originator port = %d"),
+	  data[0],data[1]);
+}
+
+static
+void sms_udh_application16_print (FILE* fp, uint8_t* data) {
+  fprintf(fp,_("Application port: destination port = %d, originator port = %d"),
+	  (data[0]<<8)+data[1],((uint16_t*)data)[1]);
+}
+
+static
+void sms_udh_source_indicator_print (FILE* fp, uint8_t* data) {
+  fprintf(fp,"%s ",_("The following headers were created by"));
+  switch (data[0]) {
+  case 1:
+    fprintf(fp,"%s",_("original sender"));
+    break;
+  case 2:
+    fprintf(fp,"%s",_("original receiver"));
+    break;
+  case 3:
+    fprintf(fp,"%s","SMSC");
+    break;
+  default:
+    fprintf(fp,_("(unknown value %d)"),data[0]);
+    break;
+  }
+}
+
+void sms_udh_print_header (FILE* fp, struct sms_pdu_ud_header* header) {
+  int k = 0;
+
+  while (sms_pdu_ud_header_defs[k].type != -1 &&
+	 sms_pdu_ud_header_defs[k].type != header->type) {
+    ++k;
+  }
+
+  if (sms_pdu_ud_header_defs[k].type != -1 &&
+      sms_pdu_ud_header_defs[k].type == header->type) {
+    fprintf(fp,"\t");
+    if (sms_pdu_ud_header_defs[k].print_func != NULL) {
+      sms_pdu_ud_header_defs[k].print_func(fp,header->data);
+    } else {
+      fprintf(fp,_("Ignored header type 0x%02x"),header->type);
+    }
+    fprintf(fp,"\n");
+  }
+}
+
+void sms_udh_print (FILE* fp, struct sms_pdu_ud* plist, uint8_t parts) {
+  struct sms_pdu_ud_header** udh = NULL;
+  int i = 0;
+  int h = 0;
+
+  if (fp == NULL || plist == NULL || parts == 0) return;
+
+  //look for at least one header
+  for (i = 0; i < parts; ++i)
+    if (plist[i].header != NULL &&
+	plist[i].header[0] != NULL) {
+      h = 1;
+      break;
+    }
+
+  if (h) fprintf(fp,"%s:\n",_("Message header"));
+  else return;
+
+  for (h = 0; h < parts; ++h) {
+    udh = plist[h].header;
+    if (udh != NULL)
+      for (i=0; udh[i] != NULL; ++i)
+	sms_udh_print_header(fp, udh[i]);
+  }
+}
+
+void sms_udh_fill (struct sms_pdu_data* sms, char* data) {
+  uint8_t udhsize = 0;
+  uint8_t offset = 0;
+  unsigned int i = 0;
+  unsigned int k;
+  unsigned int count;
+  struct sms_pdu_ud_header** header;
+
+  if (sms == NULL) {
+    return;
+  }
+  
+  if (sms->ud == NULL) {
+    sms->ud =  mem_alloc(sizeof(*(sms->ud)),0);
+    struct_sms_pdu_ud_init(sms->ud);
+  }
+
+  if (sms->options.udh_present) {
+    //extract the user data header data
+    udhsize = hexstr2int(data,2);
+    data += 2;
+    //split the udh into its parts
+    sms->ud[0].header = mem_alloc(((udhsize/2)+1)*sizeof(*sms->ud[0].header),0);
+    header = sms->ud[0].header;
+    header[i] = NULL;
+    while (offset+4 <= udhsize*2) {
+      header[i] = mem_alloc(sizeof(**header),0);
+      struct_sms_pdu_ud_header_init(header[i]);
+      //read fields that are mandatory for each user data header
+      header[i]->type = hexstr2int(data+offset,2)&0xFF;
+      header[i]->len = hexstr2int(data+2+offset,2)&0xFF;
+      offset += 4;
+      //find the udh type in the list of known headers
+      k = 0;
+      while (sms_pdu_ud_header_defs[k].type != -1 &&
+	     sms_pdu_ud_header_defs[k].type != header[i]->type)
+	++k;
+
+      //check that everything is correct with the udh data
+      if (offset+(header[i]->len*2) <= udhsize*2 &&
+	  sms_pdu_ud_header_defs[k].type != -1 &&
+	  sms_pdu_ud_header_defs[k].minlen <= header[i]->len &&
+	  header[i]->len <= sms_pdu_ud_header_defs[k].maxlen)
+      {
+	for (count = 0; count < header[i]->len; ++count) {
+	  header[i]->data[count] = hexstr2int(data+offset+2*count,2)&0xFF;
+	}
+	header[i+1] = NULL;
+	offset += 2*header[i]->len;
+      } else {
+	//bogus or unknown header, delete it and stop processing the headers
+	mem_realloc(header[i],0);
+	header[i] = NULL;
+	break;
+      }
+      ++i;
+    }
+  }
+  sms_udh_multipart_mark(sms);
+}
+
+void sms_udh_multipart_mark (struct sms_pdu_data* data) {
+  int i = 0;
+  int k;
+  struct sms_pdu_ud_header** header = NULL;
+
+  if (data == NULL) {
+    return;
+  }
+
+  if (data->ud[0].header != NULL) {
+    //save the headers
+    header = data->ud[0].header;
+    for (i=0; header[i] != NULL; ++i) {
+      if (header[i]->type == 0x00 ||
+	  header[i]->type == 0x08) {
+	if (header[i]->type == 0x00) {
+	  data->multipart_id = header[i]->data[0];
+	  data->parts = header[i]->data[1];
+	  data->partnum = header[i]->data[2]-1;
+	} else if (header[i]->type == 0x08) {
+	  data->multipart_id = *((uint16_t*)header[i]->data);
+	  data->parts = header[i]->data[2];
+	  data->partnum = header[i]->data[3]-1;
+	}
+	break;
+      }
+    }
+  }
+  data->ud = mem_realloc(data->ud,data->parts*sizeof(*(data->ud)));
+  for (k=0; k < data->parts; ++k) {
+    struct_sms_pdu_ud_init(&data->ud[k]);
+  }
+  //restore the headers to their correct position
+  data->ud[data->partnum].header = header;
+}
+
+struct sms_pdu_ud_header_def sms_pdu_ud_header_defs[] = {
   {0x00,  3,  3, sms_udh_multipart8_print}, //multipart message (8bit sequence)
   {0x01,  2,  2, sms_udh_indication_print}, //special sms indication (like in DCS)
   {0x04,  2,  2, sms_udh_application8_print}, //application port addressing 8bit address
@@ -69,232 +301,3 @@ struct sms_pdu_ud_header_def header_defs[] = {
   
   {-1,0,0, NULL} //list end element (do not remove!)
 };
-
-struct sms_pdu_ud_header*
-sms_udh_multipart16_get (uint16_t seqnum,
-			 uint8_t index,
-			 uint8_t total)
-{
-  struct sms_pdu_ud_header* retval;
-  
-  retval = mem_alloc(sizeof(*retval),0);
-  struct_sms_pdu_ud_header_init(retval);
-  retval->type = 0x08;
-  retval->len = 4;
-  retval->data = mem_alloc((4*2)+1,1);
-  sprintf(retval->data,"%02u%02u%02u",seqnum,total,index);
-  return retval;
-}
-
-void sms_udh_indication_print (FILE* fp, char* data) {
-  uint8_t octet1 = 0;
-  uint8_t octet2 = 0;
-  char* type = NULL;
-
-  octet1 = hexstr2int(data,2);
-  octet2 = hexstr2int(data+2,2);
-
-  fprintf(fp,"%s:",_("Indication"));
-  switch (octet1&0x4F) {
-  case 0: type = _("voicemail"); break;
-  case 1: type = _("fax"); break;
-  case 2: type = _("e-mail"); break;
-  case 3: type = _("misc."); break;
-  }
-  fprintf(fp,ngettext("%d %s message waiting","%d %s messages waiting",octet2),octet2,type);
-  
-}
-
-void sms_udh_wcmpdu_print (FILE* fp, char* data) {
-  fprintf(fp,"%s: %s\n",_("Wireless control message protocol data unit"),data);
-}
-
-void sms_udh_smstk_print (FILE* fp, char* data) {
-  data = data; //work around compiler warning about unused variable data
-  fprintf(fp,_("SIM toolkit security header"));
-}
-
-void sms_udh_multipart8_print (FILE* fp, char* data) {
-  fprintf(fp,_("Multipart message: sequence number = %d, part %d of %d"),
-	  hexstr2int(data,2),
-	  hexstr2int(data+4,2),
-	  hexstr2int(data+2,2));
-}
-void sms_udh_multipart16_print (FILE* fp, char* data) {
-  fprintf(fp,_("Multipart message: sequence number = %d, part %d of %d"),
-	  hexstr2int(data,4),
-	  hexstr2int(data+6,2),
-	  hexstr2int(data+4,2));
-}
-
-void sms_udh_application8_print (FILE* fp, char* data) {
-  fprintf(fp,_("Application port: destination port = %d, originator port = %d"),
-	  hexstr2int(data,2),
-	  hexstr2int(data+2,2));
-}
-
-void sms_udh_application16_print (FILE* fp, char* data) {
-  fprintf(fp,_("Application port: destination port = %d, originator port = %d"),
-	  hexstr2int(data,2),
-	  hexstr2int(data+4,2));
-}
-
-void sms_udh_source_indicator_print (FILE* fp, char* data) {
-  fprintf(fp,"%s ",_("The following headers were created by"));
-  switch (hexstr2int(data,2)) {
-  case 1:
-    fprintf(fp,"%s",_("original sender"));
-    break;
-  case 2:
-    fprintf(fp,"%s",_("original receiver"));
-    break;
-  case 3:
-    fprintf(fp,"%s",_("SMSC"));
-    break;
-  default:
-    fprintf(fp,_("(unknown value %d)"),hexstr2int(data,2));
-    break;
-  }
-}
-
-void sms_udh_print (FILE* fp, struct sms_pdu_ud* plist, uint8_t parts) {
-  struct sms_pdu_ud_header** udh = NULL;
-  int i = 0;
-  int k;
-  int h = 0;
-
-  if (fp == NULL || plist == NULL || parts == 0) {
-    return;
-  }
-
-  //look for at least one header
-  for (i = 0; i < parts; ++i) {
-    if (plist[i].header != NULL &&
-	plist[i].header[0] != NULL) {
-      h = 1;
-      break;
-    }
-  }
-
-  if (h) {
-    fprintf(fp,"%s:\n",_("Message header"));
-  } else {
-    return;
-  }
-
-  for (h = 0; h < parts; ++h) {
-    udh = plist[h].header;
-    if (udh != NULL) {
-      for (i=0; udh[i] != NULL; ++i) {
-	k=0;
-	while (header_defs[k].type != -1 &&
-	       header_defs[k].type != udh[i]->type) {
-	  ++k;
-	}
-	if (header_defs[k].type != -1 &&
-	    header_defs[k].type == udh[i]->type) {
-	  fprintf(fp,"\t");
-	  if (header_defs[k].print_func != NULL) {
-	    header_defs[k].print_func(fp,udh[i]->data);
-	  } else {
-	    fprintf(fp,_("Ignored header type 0x%02x"),udh[i]->type);
-	  }
-	  fprintf(fp,"\n");
-	}
-      }
-    }
-  }
-}
-
-void sms_udh_fill (struct sms_pdu_data* sms, char* data) {
-  uint8_t udhsize = 0;
-  uint8_t offset = 0;
-  unsigned int i = 0;
-  unsigned int k;
-  struct sms_pdu_ud_header** header;
-
-  if (sms == NULL) {
-    return;
-  }
-  
-  if (sms->ud == NULL) {
-    sms->ud =  mem_alloc(sizeof(*(sms->ud)),0);
-    struct_sms_pdu_ud_init(sms->ud);
-  }
-
-  if (sms->options.udh_present) {
-    //extract the user data header data
-    udhsize = hexstr2int(data,2);
-    //split the udh into its parts
-    sms->ud[0].header = mem_alloc(((udhsize/2)+1)*sizeof(*sms->ud[0].header),0);
-    header = sms->ud[0].header;
-    header[i] = NULL;
-    while (offset+4 <= udhsize*2) {
-      header[i] = mem_alloc(sizeof(**header),0);
-      struct_sms_pdu_ud_header_init(header[i]);
-      //read fields that are mandatory for each user data header
-      header[i]->type = hexstr2int(data+2+offset,2)&0xFF;
-      header[i]->len = hexstr2int(data+4+offset,2)&0xFF;
-      offset += 4;
-      //find the udh type in the list of known headers
-      k = 0;
-      while (header_defs[k].type != -1 &&
-	     header_defs[k].type != header[i]->type) {
-	++k;
-      }
-      //check that everything is correct with the udh data
-      if (offset+(header[i]->len*2) <= udhsize*2 &&
-	  header_defs[k].type != -1 &&
-	  header_defs[k].minlen <= header[i]->len &&
-	  header[i]->len <= header_defs[k].maxlen) {
-	header[i]->data = strn_dup(data+2+offset, header[i]->len*2);
-	offset += header[i]->len*2;
-	header[i+1] = NULL;
-      } else {
-	//bogus or unknown header, delete it and stop processing the headers
-	struct_sms_pdu_ud_header_delete(header[i]);
-	mem_realloc(header[i],0);
-	header[i] = NULL;
-	break;
-      }
-      ++i;
-    }
-  }
-  sms_udh_multipart_mark(sms);
-}
-
-void sms_udh_multipart_mark (struct sms_pdu_data* data) {
-  int i = 0;
-  int k;
-  struct sms_pdu_ud_header** header = NULL;
-
-  if (data == NULL) {
-    return;
-  }
-
-  if (data->ud[0].header != NULL) {
-    //save the headers
-    header = data->ud[0].header;
-    for (i=0; header[i] != NULL; ++i) {
-      if (header[i]->type == 0x00 ||
-	  header[i]->type == 0x08) {
-	if (header[i]->type == 0x00) {
-	  data->multipart_id = hexstr2int(header[i]->data,2);
-	  data->parts = hexstr2int(header[i]->data+2,2);
-	  data->partnum = hexstr2int(header[i]->data+4,2)-1;
-	} else if (header[i]->type == 0x08) {
-	  data->multipart_id = hexstr2int(header[i]->data,4);
-	  data->parts = hexstr2int(header[i]->data+4,2);
-	  data->partnum = hexstr2int(header[i]->data+6,2)-1;
-	}
-	break;
-      }
-    }
-  }
-  data->ud = mem_realloc(data->ud,data->parts*sizeof(*(data->ud)));
-  for (k=0; k < data->parts; ++k) {
-    struct_sms_pdu_ud_init(&data->ud[k]);
-  }
-  //restore the headers to their correct position
-  data->ud[data->partnum].header = header;
-}
