@@ -20,36 +20,33 @@ struct sms_pdu_ud_header* sms_udh_multipart16_get (uint16_t seqnum,
 						   uint8_t total);
 
 
-/* print the headers as text to fp
- * sms_udh_fill() and sms_multipart_merge() should be
- * called before this one
+/** print all headers as text to fp
+ *  sms_udh_fill() and sms_multipart_merge() should be
+ *  called before this one
+ *  This is called e.g. by sms_pdu_print_deliver()
  *
- * called e.g. by sms_pdu_print_deliver()
+ *  @param fp file/pipe/etc. to print to
+ *  @param plist user data from the (concatenated) short message
+ *  @param parts message parts (1-255)
  */
 void sms_udh_print (FILE* fp, struct sms_pdu_ud* plist, uint8_t parts);
+void sms_udh_print_header (FILE* fp, struct sms_pdu_ud_header* header);
 
-//these are called by sms_udh_print()
-void sms_udh_multipart8_print (FILE* fp, char* data);
-void sms_udh_multipart16_print (FILE* fp, char* data);
-void sms_udh_application8_print (FILE* fp, char* data);
-void sms_udh_application16_print (FILE* fp, char* data);
-void sms_udh_source_indicator_print (FILE* fp, char* data);
-void sms_udh_smstk_print (FILE* fp, char* data);
-void sms_udh_wcmpdu_print (FILE* fp, char* data);
-void sms_udh_indication_print (FILE* fp, char* data);
+/** to parse the user data header from a pdu
+ */
+void sms_udh_fill (struct sms_pdu_data* sms, char* data);
+
+/** called by sms_udh_fill(), no need to call anywhere else
+ */
+void sms_udh_multipart_mark (struct sms_pdu_data* data);
 
 //mainly used by sms_udh_fill()
 struct sms_pdu_ud_header_def {
   int16_t type; //negative entries are invalid
   uint8_t minlen; //in octets
   uint8_t maxlen; //in octets (obviously not more than 137(=140-1-2)))
-  void (*print_func) (FILE* fp, char* data); //function pointer to header type printer
+  void (*print_func) (FILE* fp, uint8_t* data); //function pointer to header type printer
 };
-extern struct sms_pdu_ud_header_def header_defs[]; //known header types
-
-//to parse the user data header from a pdu
-void sms_udh_fill (struct sms_pdu_data* sms, char* data);
-//called by sms_udh_fill(), no need to call anywhere else
-void sms_udh_multipart_mark (struct sms_pdu_data* data);
+extern struct sms_pdu_ud_header_def sms_pdu_ud_header_defs[]; //known header types
 
 #endif
