@@ -14,6 +14,7 @@
 #include "helper.h"
 #include "atcommand.h"
 #include "config.h" //needed for TTYSPEED
+#include "gtincl.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -27,7 +28,7 @@
 #include <unistd.h>
 
 #ifndef OS2
-int tty_portfd;
+static int tty_portfd;
 #else
 #  define  INCL_OS2
 #  define  INCL_DOSDEVICES
@@ -48,7 +49,7 @@ speed_t tty_speed(char *newttyspeed){
   case 115200: return B115200;
 #endif
   default:
-    myprintf(0,"%s","No valid baudrate defined. Using compiled in value...\n");
+    print_verbose(0,"%s","No valid baudrate defined. Using compiled in value...\n");
     return tty_speed(TTYSPEED);
   }
 }
@@ -68,48 +69,116 @@ int cfmakeraw(struct termios *termios_p) {
 }
 #endif
 
+void tty_flush () {
+  tcflush(tty_portfd, TCIOFLUSH); //clear serial tty
+}
+
 #define NONBLOCK_OPEN 1
 void tty_open(struct port_args_t* args) {
-#ifndef OS2
+#ifdef OS2
+  LINECONTROL lc;
+  long action,size;
+  int bps,rc,timeout;
+  DCBINFO devinfo;
+  
+  //opening the device
+  print_verbose(0,_("Accessing device %s..."),args->device);
+  rc = DosOpen(args->device,&tty_portfd, &action, 0, FILE_NORMAL, FILE_OPEN,
+	       OPEN_ACCESS_READWRITE|OPEN_SHARE_DENYNONE,
+	       (PEAOP2)NULL);
+  
+  if (rc) {
+    print_verbose(0,_("Cannot open %s: rc=%d\n"),args->device,rc);
+    exit(EXIT_FAILURE);
+  }
+  // setting baud rate
+  bps = tty_speed(args->baud);
+  rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETBAUDRATE,
+		   &bps, sizeof(ULONG), &size, NULL, 0, NULL);
+  if (rc) {
+    fprintf(stderr,"%s: rc= %d\n",_("Error in setting transmission speed"),rc);
+    exit(EXIT_FAILURE);
+  }
+  // set 8,N,1
+  lc.bDataBits = 8;
+  lc.bParity = 0;
+  lc.bStopBits = 0;
+  lc.fTransBreak = 0;
+  rc = DosDevIOCtl(tty_portfd,
+		   IOCTL_ASYNC, ASYNC_SETLINECTRL,
+		   &lc, sizeof(LINECONTROL), &size,
+		   NULL, 0, NULL);
+  if (rc) {
+    fprintf(stderr,"%s: rc= %d\n",_("Error in setting port attributes"),rc);
+    exit(EXIT_FAILURE);
+  }
+
+  size = sizeof(DCBINFO);
+  rc = DosDevIOCtl(tty_portfd, /* file decsriptor */
+		   IOCTL_ASYNC, /*asyncronous change */
+		   ASYNC_GETDCBINFO, /* get device control block info */
+		   NULL, /*  */
+		   0, /* length of the previous parameter */
+		   NULL, /* max length of data ret */
+		   &devinfo, /* data to be recieved */
+		   size, /* length of data */
+		   (unsigned long *)&size); /* length of data returned */
+  
+  if (args->timeout == 0) { // infinity waiting
+    timeout = 65535;
+  } else {
+    timeout = args->timeout*10;
+    if (timeout > 65535) timeout = 65535;
+  } 	  
+  devinfo.fbTimeout = 2;
+  devinfo.fbFlowReplace = 0; // diable rts/cts control
+  devinfo.fbCtlHndShake = 1; // Truning on DTR.
+  devinfo.usReadTimeout = timeout;
+  devinfo.usWriteTimeout = timeout;
+
+  size = sizeof(DCBINFO);
+  rc = DosDevIOCtl(tty_portfd, 
+		   IOCTL_ASYNC, 
+		   ASYNC_SETDCBINFO,
+		   &devinfo, /* parameters to set  */
+		   size,
+		   (unsigned long *)&size, 
+		   NULL, 
+		   0,
+		   NULL); 
+
+#else
   int flags = O_RDWR | O_NOCTTY;
   struct termios newtio;
-#else //OS2
-  LINECONTROL lc;
-#endif
-  char port[1024];
 
-  //finding real device name
-  memset(port,0,sizeof(port));
-  strncpy(port,args->device,sizeof(port));
-  //opening the device
-  if (strcmp(args->device,port) == 0) {
-    myprintf(0,"Accessing device %s...",port);
-  } else {
-    myprintf(0,"Accessing device %s -> %s...",args->device,port);    
-  }
-#ifndef OS2  
+#if NONBLOCK_OPEN
+  flags |= O_NONBLOCK;
+#endif
+
   /* FHS (http://www.pathname.com/fhs/pub/fhs-2.3.html#VARLOCKLOCKFILES)
    * specifies lock files for devices but the Serial HowTo (13.3)
    * also mentions the problem when it comes to DevFS (Linux).
    * Since this makes it an unreliable test, forget about it.
    */
-#if NONBLOCK_OPEN
-  flags |= O_NONBLOCK;
-#endif
-  if ((tty_portfd=open(port,flags)) == -1) {
-    myprintf(0,"Cannot open %s: %s\n",port,strerror(errno));
+
+  //opening the device
+  print_verbose(0,_("Accessing device %s..."),args->device);
+  if ((tty_portfd=open(args->device,flags)) == -1) {
+    print_verbose(0,_("Cannot open %s: %s\n"),args->device,strerror(errno));
     exit(EXIT_FAILURE);
   }
+
 #if NONBLOCK_OPEN
   if (fcntl(tty_portfd,F_SETFL,flags&(~O_NONBLOCK)) == -1) {
-    fprintf(stderr,"Error in setting port attributes: %s\n",strerror(errno));
+    fprintf(stderr,"%s: %s\n",_("Error in setting port attributes"),strerror(errno));
     exit(EXIT_FAILURE);
   }
-#endif //NONBLOCK_OPEN
+#endif
+
   if (!args->ignorebits) {
     //getting port parameters
     if (tcgetattr(tty_portfd,&newtio) < 0){
-      fprintf(stderr,"Error in getting device attributes: %s\n",strerror(errno));
+      fprintf(stderr,"%s: %s\n",_("Error in getting port attributes"),strerror(errno));
       exit(EXIT_FAILURE);
     }
   } else {
@@ -130,97 +199,41 @@ void tty_open(struct port_args_t* args) {
     newtio.c_cc[VMIN]=0; //return after timeout, even with nothing
   }
   newtio.c_cc[VTIME]=args->timeout; //try reading for this amount of time (in deciseconds)
-  tcflush(tty_portfd, TCIOFLUSH); //clear serial tty
+  tty_flush();
   if(tcsetattr(tty_portfd,TCSANOW,&newtio) < 0){ //set now
-    fprintf(stderr,"Error in setting device attributes: %s\n",strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-#else //OS2
-  long action,size;
-  int bps,rc,timeout;
-  DCBINFO devinfo;
-
-
-  rc = DosOpen(port,&tty_portfd, &action, 0, FILE_NORMAL, FILE_OPEN,
-	       OPEN_ACCESS_READWRITE|OPEN_SHARE_DENYNONE,
-	       (PEAOP2)NULL);
-  
-  if (rc) {
-    myprintf(0,"Cannot open %s: rc=%d\n",port,rc);
-    exit(EXIT_FAILURE);
-  }
-  // setting baud rate
-  bps = tty_speed(args->baud);
-  rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETBAUDRATE,
-		   &bps, sizeof(ULONG), &size, NULL, 0, NULL);
-  if (rc) {
-    fprintf(stderr,"Error in setting device speed:rc= %d\n",rc);
-    exit(EXIT_FAILURE);
-  }
-  // set 8,N,1
-  lc.bDataBits = 8;
-  lc.bParity = 0;
-  lc.bStopBits = 0;
-  lc.fTransBreak = 0;
-  rc = DosDevIOCtl(tty_portfd,
-		   IOCTL_ASYNC, ASYNC_SETLINECTRL,
-		   &lc, sizeof(LINECONTROL), &size,
-		   NULL, 0, NULL);
-  if (rc) {
-    fprintf(stderr,"Error in setting device attributes:rc= %d\n",rc);
+    fprintf(stderr,"%s: %s\n",_("Error in setting port attributes"),strerror(errno));
     exit(EXIT_FAILURE);
   }
 
-   size=sizeof(DCBINFO);
-   rc=DosDevIOCtl(tty_portfd, /* file decsriptor */
-		 IOCTL_ASYNC, /*asyncronous change */
-		 ASYNC_GETDCBINFO, /* get device control block info */
-		 NULL, /*  */
-		 0, /* length of the previous parameter */
-		 NULL, /* max length of data ret */
-		 &devinfo, /* data to be recieved */
-		 size, /* length of data */
-		 (unsigned long *)&size); /* length of data returned */
-   
-  if (args->timeout == 0) { // infinity waiting
-	timeout=65535;
-  } else {
-	timeout=args->timeout*10;
-	if (timeout>65535)
-	    timeout=65535;
-  } 	  
-  devinfo.fbTimeout = 2;
-  devinfo.fbFlowReplace = 0; // diable rts/cts control
-  devinfo.fbCtlHndShake = 1; // Truning on DTR.
-  devinfo.usReadTimeout=timeout;
-  devinfo.usWriteTimeout=timeout;
+#endif
 
-   
-
-   size=sizeof(DCBINFO);
-   rc=DosDevIOCtl(tty_portfd, 
-		 IOCTL_ASYNC, 
-		 ASYNC_SETDCBINFO,
-		 &devinfo, /* parameters to set  */
-		 size,
-		 (unsigned long *)&size, 
-		 NULL, 
-		 0,
-		 NULL); 
- // sleep(1); /* i dont know why but without this delay it is unusable :( */
-#endif //OS2
-  myprintf(0,"%s","done\n");
+  print_verbose(0,"%s\n",_("done"));
   /* The following seems to be needed for C45
    * which seems to react too slow
    */
   if (args->startdelay) {
-    myprintf(0,"Waiting for %d seconds as requested...\n",args->startdelay);
+    print_verbose(0,_("Waiting for %d seconds as requested...\n"),args->startdelay);
     sleep(args->startdelay);
   }
 }
 
 int tty_write (const char* data, size_t count) {
-#ifndef OS2
+#ifdef OS2
+  int rc;
+  ULONG status = 0;
+  ULONG instatus;
+
+  do { 
+    rc = DosWrite(tty_portfd,&data[status],count-status,&instatus);
+    if (!rc) {
+      status += instatus;
+    } else {
+      fprintf(stderr,"Error in DosWrite: rc= %d\n",rc);
+      return 0;
+    }
+  } while (count-status > 0);
+
+#else
   ssize_t status = 0;
   ssize_t instatus;
 
@@ -233,19 +246,7 @@ int tty_write (const char* data, size_t count) {
       else return 0;
     }
   } while (count-status > 0);
-#else
-  int rc;
-  ULONG status = 0;
-  ULONG instatus;
 
-  do { 
-    rc = DosWrite(tty_portfd,&data[status],count-status,&instatus);
-    if(!rc) status += instatus;
-    else {
-	 fprintf(stderr,"Error in DosWrite: rc= %d\n",rc);
-     return 0;
-    }
-  } while (count-status > 0);
 #endif
   return 1;
 }
@@ -257,23 +258,38 @@ int tty_write (const char* data, size_t count) {
  * (the timeout might only happen when the device was not opened with O_NONBLOCK,
  *  the timeout time refers to the value that was set as c_cc[VTIME])
  */
-char* tty_read_line (int (*stop_condition)(const char*,const size_t))
-{	
+char* tty_read_line (int (*stop_condition)(const char*,const size_t)) {	
   char* retval;
   int retvalsize = 0;
-#ifndef OS2
-  int status;
-#else
-  ULONG status;
-  int rc;
-#endif
   char buffer; //buffer the read character for checks
   int counter = 0; //count the read-and-used characters
   int repeat = 0; //tell the inner loop to loop
 
+#ifdef OS2
+  ULONG status;
+  int rc;
+
+  retval = NULL;
+  do {
+    do {
+      rc = DosRead(tty_portfd,&buffer,1,&status); /* expect a single byte */
+      if (rc) {
+	fprintf(stderr, "Serial RX: timeout\n");
+	mem_realloc(retval,0);
+	return NULL;
+      }
+      usleep(1);
+      if (status==0) {
+	mem_realloc(retval,0);
+	return str_dup("");
+      }
+    } while (repeat);
+
+#else
+  int status;
+
   retval = NULL;
   do {
-#ifndef OS2	  
     do {
       status = read(tty_portfd,&buffer,1);
       switch (status) {
@@ -292,21 +308,8 @@ char* tty_read_line (int (*stop_condition)(const char*,const size_t))
 	repeat = 0;
       }
     } while (repeat);
-#else //OS2
-    do {
-      rc = DosRead(tty_portfd,&buffer,1,&status); /* expect a single byte */
-      if (rc) {
-	fprintf(stderr, "Serial RX: timeout\n");
-	mem_realloc(retval,0);
-	return NULL;
-      }
-      usleep(1);
-      if (status==0) {
-	mem_realloc(retval,0);
-	return str_dup("");
-      }
-    } while (repeat);
-#endif //OS2
+
+#endif
 
     // allocate space on stack
     if (counter >= retvalsize) {
@@ -329,9 +332,11 @@ char* tty_read_line (int (*stop_condition)(const char*,const size_t))
 }
 
 void tty_close(){
-#ifndef OS2
-  close(tty_portfd);
-#else
+#ifdef OS2
   DosClose(tty_portfd); 
+
+#else
+  close(tty_portfd);
+
 #endif
 }
