@@ -9,31 +9,45 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "options.h"
-#include <termios.h>
+#ifndef TTYACCESS_H
+#define TTYACCESS_H
 
+#include "intincl.h"
+
+#include <termios.h>
 #include <sys/types.h>
 
+struct port_args_t {
+  char* device;
+  char* baud;
+  uint8_t timeout;
+  unsigned int ignorebits :1;
+  unsigned int startdelay;
+};
+
 void tty_open (struct port_args_t* args);
 void tty_close();
 
 #ifdef OS2
 #define speed_t int
-#undef B9600
-#undef B19200
-#undef B38400
-#undef B57600
-#undef B115200
+#undef  B9600
 #define B9600 9600
+#undef  B19200
 #define B19200 19200
+#undef  B38400
 #define B38400 38400
+#undef  B57600
 #define B57600 57600
+#undef  B115200
 #define B115200 115200
 #endif
 speed_t tty_speed(char *newttyspeed);
 
 int  tty_write (const char* data, size_t count);
 char* tty_read_line (int (*stop_condition)(const char*,const size_t));
+void tty_flush ();
 
 // \x1a represents Ctrl-Z (represents end of data stream)
 #define tty_write_data(data,count) (tty_write(data,count) && tty_write("\x1a",1))
+
+#endif
